Package: SharedObject
Type: Package
Title: Sharing R objects across multiple R processes without memory
        duplication
Version: 1.26.2
Date: 2023-8-9
Authors@R: c(
    person(
        "Jiefei", "Wang",
        email = "szwjf08@gmail.com",
        role = c("aut", "cre")
    ),
    person("Martin", "Morgan", role = "aut"))
Description: This package is developed for facilitating parallel
        computing in R. It is capable to create an R object in the
        shared memory space and share the data across multiple R
        processes. It avoids the overhead of memory dulplication and
        data transfer, which make sharing big data object across many
        clusters possible.
License: GPL-3
LinkingTo: BH, Rcpp
Depends: R (>= 3.6.0)
Imports: Rcpp, methods, stats, BiocGenerics
biocViews: Infrastructure
BugReports: https://github.com/Jiefei-Wang/SharedObject/issues
Suggests: testthat, parallel, knitr, rmarkdown, BiocStyle
RoxygenNote: 7.1.1
Roxygen: list(markdown = TRUE)
VignetteBuilder: knitr
SystemRequirements: GNU make, C++11
Encoding: UTF-8
Config/pak/sysreqs: make
Repository: Bioconductor 3.23
Date/Publication: 2026-08-12 14:55:28 UTC
RemoteUrl: https://github.com/bioc/SharedObject
RemoteRef: RELEASE_3_23
RemoteSha: 62f14b1fce30499112be8b9040021368748b765f
NeedsCompilation: yes
Packaged: 2026-08-12 21:35:52 UTC; root
Author: Jiefei Wang [aut, cre],
  Martin Morgan [aut]
Maintainer: Jiefei Wang <szwjf08@gmail.com>
Built: R 4.6.1; aarch64-apple-darwin23; 2026-08-12 21:37:43 UTC; unix

# Generated by roxygen2: do not edit by hand

export(SharedObject)
export(allocateSharedMemory)
export(freeSharedMemory)
export(getCopyOnWrite)
export(getLastIndex)
export(getSharedCopy)
export(getSharedMemorySize)
export(getSharedSubset)
export(hasSharedMemory)
export(initialSharedObjectPackageData)
export(is.shared)
export(listSharedObjects)
export(mapSharedMemory)
export(releaseSharedObjectPackageData)
export(setCopyOnWrite)
export(setSharedCopy)
export(setSharedSubset)
export(share)
export(sharedObjectPkgOptions)
export(sharedObjectProperties)
export(unmapSharedMemory)
export(unshare)
exportMethods(is.shared)
exportMethods(share)
exportMethods(sharedObjectProperties)
exportMethods(unshare)
import(BiocGenerics)
import(methods)
importFrom(Rcpp,sourceCpp)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(stats,runif)
useDynLib(SharedObject, .registration = TRUE)

doList <- function(func,tryFunc, x, ...){
    options <- completeOptions(...)
    ## We do not create a new list unless
    ## there exists any sharable elements
    for(i in seq_along(x)){
        Elt <- func(x[[i]],...)
        if(!C_isSameObject(x[[i]],Elt)){
            x[[i]] <- Elt
        }
    }
    if(options$sharedAttributes){
        ## share attributes
        oldAttrs <- attributes(x)
        if(!is.null(oldAttrs)){
            ## Exclude names and class attributes.
            ## If attributes(x) contains the names attribute,
            ## attributes(attributes(x)) still contains the names attributes
            ## and will cause an infinit loop. We must ignore it.
            ## The later one is not worthy to share
            if(!all(names(oldAttrs) %in% c("class","names"))){
                attrs <- doAttributes(tryFunc, oldAttrs, ...)
                ## set the attributes only when it is different from the old one
                if(!C_isSameObject(oldAttrs,attrs)){
                    attributes(x) <- attrs
                }
            }
        }
    }
    x
}
doS4 <- function(func, x, ...){
    ## If the object is an S4SXP,
    ## share its slots
    if(isSharableAtomic(x)||isSEXPList(x)){
        ## If the object is not an S4SXP,
        ## Calling the right share method
        C_UNSETS4(x)
        if(isS4(x))
            stop("Unexpected error, cannot convert an S4 object to a non-S4 object")
        result <- func(x,...)
        C_SETS4(x)
        C_SETS4(result)
    }else{
        result <- x
        slots <- slotNames(result)
        for(i in slots){
            slot(result, i, check = FALSE) <- func(slot(x, i), ...)
        }
    }
    validObject(result)
    result
}

doEnvironment <- function(func,tryFunc, x, ...){
    options <- completeOptions(...)
    for(i in names(x)){
        x[[i]] <- func(x[[i]],...)
    }
    if(options$sharedAttributes){
        attributes(x) <- tryFunc(attributes(x),...)
    }
    x
}



doAttributes<-function(tryFunc, attrs, ...){
    attrNames <- names(attrs)
    idx <- which(!attrNames%in%c("class","names","levels"))
    for(i in idx){
        newAttri <- tryFunc(attrs[[i]], ...)
        if(!C_isSameObject(newAttri,attrs[[i]])){
            attrs[[i]] <- newAttri
        }
    }
    attrs
}

#' @useDynLib SharedObject, .registration = TRUE
#' @importFrom Rcpp sourceCpp
#' @importFrom stats runif
#' @importFrom methods is new as
#' @import BiocGenerics
#' @import methods
NULL

errorHandler<- function(e){
    msg <- conditionMessage(e)
    if(getOS()=="osx"){
        msg <- paste0(msg, 
                      "\n If the error is 'Operation not permitted', you might not have permission to access '/tmp/boost_interprocess'")
    }
    warning(msg)
}


.onLoad <- function(libname, pkgname){
    if(Sys.getenv("DEBUG_SHARED_OBJECT_PACKAGE")==""){
        setVerbose(FALSE)
    }else{
        C_setSharedMemoryPrint(TRUE)
        C_setAltrepPrint(FALSE)
        C_setPackagePrint(TRUE)
    }
    tryCatch({
        initialSharedObjectPackageData()
    }, error = errorHandler)
    initialSharedObjectPackageData()
}

#' Create an empty shared object
#'
#' Create an empty shared object with a specific length and attributes.
#'
#' @param mode the type of the shared object
#' @param length the length of the shared object
#' @param attrib the attributes of the shared object
#' @param ... Parameters that is used to create the shared object, please
#' refer to `?share` for details.
#' @examples
#' ## Create an empty shared vector
#' x1 <- SharedObject(mode = "numeric", length = 10)
#' x1
#' ## Create an empty shared matrix
#' x2 <- SharedObject(mode = "numeric", length = 6,
#'                  attrib = list(dim = c(2L,3L)))
#' x2
#' @return An R vector
#' @export
SharedObject <- function(mode = c("raw","logical","integer","numeric","complex"),
                       length, attrib = list(), ...) {
    mode <- match.arg(mode)
    options <- completeOptions(...)
    if(options$sharedAttributes){
        attrib <- doAttributes(tryShare, attrib,...)
    }
    result <- C_createEmptySharedObject(
        type = C_getDataTypeId(mode),
        length= length,
        copyOnWrite=options$copyOnWrite,
        sharedSubset=options$sharedSubset,
        sharedCopy=options$sharedCopy,
        attributes=as.pairlist(attrib)
    )
    result
}

context("the ShareObject function")
sharedObjectPkgOptions(minLength = 10)

types <- c("raw", "logical", "integer", "numeric", "complex")

for(type in types){
    test_that("0-length vector", {
        n <- 0
        x <- SharedObject(mode = type, length = n)
        expect_equal(length(x), n)
        expect_equal(type, class(x))
        expect_true(is.shared(x))
        expect_equal(x, vector(mode = type, length = n))
    })


    test_that("create empty shared vector but length smaller than minLenght",{
        n <- 5
        x <- SharedObject(mode = type, length = n)
        expect_equal(length(x), n)
        expect_equal(type, class(x))
        expect_true(is.shared(x))
        expect_equal(x, vector(mode = type, length = n))
    })

    test_that("create empty shared vector",{
        n <- 20
        x <- SharedObject(mode = type, length = n)
        expect_equal(length(x), n)
        expect_equal(type, class(x))
        expect_true(is.shared(x))
        expect_equal(x, vector(mode = type, length = n))
    })
}

globalSettings = new.env()
globalSettings$copyOnWrite = TRUE
globalSettings$sharedSubset = FALSE
globalSettings$sharedCopy = FALSE
globalSettings$sharedAttributes = TRUE
globalSettings$minLength = 3L
globalSettings$mustWork = FALSE

#' Get or set the global options for the SharedObject package
#'
#' @param ... The name of the option(s), it can be either symbols or characters.
#' if the argument is missing, it means getting all option. See examples.
#' @param literal Whether the parameters in `...` are always treated as characters.
#'
#' @return
#' `set`: The old package options
#'
#' `get`: A list of the package options or a single value
#' @examples
#' ## Get all options
#' sharedObjectPkgOptions()
#'
#' ## Get copyOnWrite only
#' sharedObjectPkgOptions(copyOnWrite)
#' sharedObjectPkgOptions("copyOnWrite")
#' opt <- "copyOnWrite"
#' sharedObjectPkgOptions(opt, literal = FALSE)
#'
#'
#' ## Set options
#' sharedObjectPkgOptions(copyOnWrite = FALSE)
#' ## Check if we have changed the option
#' sharedObjectPkgOptions(copyOnWrite)
#'
#' ## Restore the default
#' sharedObjectPkgOptions(copyOnWrite = TRUE)
#' @rdname sharedObjectPkgOptions
#' @export
sharedObjectPkgOptions <- function(..., literal = TRUE){
    sysCall <- as.list(sys.call())[-1]
    args <- processArgs(sysCall,literal)
    if(length(args$argsGetCommand)!=0&&
       length(args$argsSetCommand)!=0){
        stop("You cannot get and set the package settings at the same time")
    }
    if(length(args$argsSetCommand)!=0){
        setSharedObjectPkgOptions(args$argsSetCommand)
    }else{
        getSharedObjectPkgOptions(args$argsGetCommand)
    }
}
setSharedObjectPkgOptions <- function(options) {
    options = checkOptionExistance(options)
    oldOptions <- as.list(globalSettings)[names(options)]
    for (i in seq_along(options)) {
        globalSettings[[names(options)[i]]] = options[[i]]
    }
    if(length(options)==1){
        oldOptions <- unlist(oldOptions)
    }
    invisible(oldOptions)
}

getSharedObjectPkgOptions <- function(options) {
    if (length(options) == 0) {
        return(as.list(globalSettings))
    } else{
        if(length(options)==1){
            return(globalSettings[[options]])
        }else{
            return(as.list(globalSettings)[options])
        }
    }
}

## Check if options exist or not
## return the options that exist
checkOptionExistance <- function(options) {
    noneExistOptions = !names(options) %in% names(globalSettings)
    if (any(noneExistOptions)) {
        vapply(paste0(names(options)[noneExistOptions]), function(x)
            warning(paste0("The option `", x, "` does not exist")), character(1))
    }
    options = options[!noneExistOptions]
    options
}

## Fill the options with their default argument
## if not specified
completeOptions <- function(...) {
    options <- list(...)
    defaultOptions <- sharedObjectPkgOptions()
    ind <- !names(defaultOptions) %in% names(options)
    c(options,defaultOptions[ind])
}

context("Shared vector memory burden test")
sharedObjectPkgOptions(minLength = 1)

library(parallel)
cl = makeCluster(2)


N <- 100
n<- 1000
for(i in seq_len(N)){
    test_that(paste0("Testing small memory alloc/free: ", i ),
              {
                  mydata <- runif(n)
                  so = share(mydata)
                  clusterExport(cl, "so", envir = environment())
                  res = clusterEvalQ(cl, {
                      so[1:10]
                  })
                  expect_equal(mydata[1:10], res[[1]][1:10])
                  res = clusterEvalQ(cl, {
                      rm(list="so")
                      gc()
                  })
                  rm(list="so")
                  gc()
              })
}
stopCluster(cl)
gc()

cl = makeCluster(2)
## use 128MB + 128MB each time
## If not shared, use 4*128MB in total each time
N <- 10
n<- round(128*1024*1024/8)

for(i in seq_len(N)){
    test_that(paste0("Testing big memory alloc/free: ", i),
              {
                  mydata <- runif(n)
                  so = share(mydata)
                  clusterExport(cl, "so", envir = environment())
                  res = clusterEvalQ(cl, {
                      so[1:10]
                  })
                  expect_equal(mydata[1:10], res[[1]][1:10])
                  res = clusterEvalQ(cl, {
                      rm(list="so")
                      gc()
                  })
                  rm(list="so")
                  gc()
              })

}
stopCluster(cl)
gc()

context("Share character vector")
sharedObjectPkgOptions(minLength = 0)

test_that("zero-length vector",{
    x1 <- character(0)
    x2 <- share(x1)
    expect_true(is.shared(x2))
    expect_equal(x1, x2)
    props <- sharedObjectProperties(x2)
    expect_equal(props$length, length(x1))
    expect_equal(props$unitSize, 1)
    expect_equal(props$totalSize, length(x1)*props$unitSize)
    expect_equal(props$uniqueChar, length(unique(x1)))
})


test_that("length 1 vector",{
    x1 <- character(1)
    x2 <- share(x1)
    expect_true(is.shared(x2))
    expect_equal(x1, x2)
    props <- sharedObjectProperties(x2)
    expect_equal(props$length, length(x1))
    expect_equal(props$unitSize, 1)
    expect_equal(props$totalSize, length(x1)*props$unitSize)
    expect_equal(props$uniqueChar, length(unique(x1)))
})


test_that("long vector with one unique element",{
    x1 <- character(1000)
    x2 <- share(x1)
    expect_true(is.shared(x2))
    expect_equal(x1, x2)
    props <- sharedObjectProperties(x2)
    expect_equal(props$length, length(x1))
    expect_equal(props$unitSize, 1)
    expect_equal(props$totalSize, length(x1)*props$unitSize)
    expect_equal(props$uniqueChar, length(unique(x1)))
})


test_that("long vector with all unique element",{
    x1 <- as.character(1:256)
    x2 <- share(x1)
    expect_true(is.shared(x2))
    expect_equal(x1, x2)
    props <- sharedObjectProperties(x2)
    expect_equal(props$length, length(x1))
    expect_equal(props$unitSize, 1)
    expect_equal(props$totalSize, length(x1)*props$unitSize)
    expect_equal(props$uniqueChar, length(unique(x1)))


    x1 <- as.character(1:257)
    x2 <- share(x1)
    expect_true(is.shared(x2))
    expect_equal(x1, x2)
    props <- sharedObjectProperties(x2)
    expect_equal(props$length, length(x1))
    expect_equal(props$unitSize, 2)
    expect_equal(props$totalSize, length(x1)*props$unitSize)
    expect_equal(props$uniqueChar, length(unique(x1)))
})

context("memory management")
sharedObjectPkgOptions(minLength = 1)
library(parallel)

test_that("Manage a shared object at current process", {
    x <- share(1:10)
    gc()
    info <- sharedObjectProperties(x)
    expect_true(hasSharedMemory(info$dataId))
    rm(x)
    gc()
    expect_false(hasSharedMemory(info$dataId))
})

test_that("release a shared object when it is in a child process", {
    cl <- makeCluster(1)
    x <- share(1:10)
    info <- sharedObjectProperties(x)
    expect_true(hasSharedMemory(info$dataId))
    clusterExport(cl, "x", envir = environment())
    gc()
    ## Check equality
    expect_equal(clusterEvalQ(cl, x)[[1]], 1:10)
    gc()
    expect_equal(x, clusterEvalQ(cl, x)[[1]])
    gc()
    expect_true(hasSharedMemory(info$dataId))
    ## release it from the main process
    rm(x)
    gc()

    if(getOS() == "windows"){
        expect_true(hasSharedMemory(info$dataId))
        ## release it from another process
        clusterEvalQ(cl,{rm(x);gc()})
        gc()
        expect_false(hasSharedMemory(info$dataId))
    }else{
        expect_false(hasSharedMemory(info$dataId))
        expect_warning(clusterEvalQ(cl,x))
        expect_error(clusterEvalQ(cl,{rm(x);gc()}),NA)
    }
    expect_error(clusterEvalQ(cl,{gc()}), NA)
    stopCluster(cl)
})


test_that("release a shared object from a child process", {
    cl <- makeCluster(1)
    x <- share(1:10)
    info <- sharedObjectProperties(x)
    expect_true(hasSharedMemory(info$dataId))
    clusterExport(cl, "x", envir = environment())
    gc()
    ## Check equality
    expect_equal(clusterEvalQ(cl, x)[[1]], 1:10)
    gc()
    expect_equal(x, clusterEvalQ(cl, x)[[1]])
    gc()
    expect_true(hasSharedMemory(info$dataId))
    ## release it from the child process
    clusterEvalQ(cl,{rm(x);gc()})
    expect_true(hasSharedMemory(info$dataId))
    ## release it from the main process
    rm(x)
    gc()
    expect_false(hasSharedMemory(info$dataId))
    stopCluster(cl)
})


test_that("create a shared object from a child process then remove from child first", {
    cl <- makeCluster(1)
    x <- clusterEvalQ(cl, {
        library(SharedObject)
        x <- share(1:10)
        x
    })[[1]]
    info <- sharedObjectProperties(x)
    expect_true(hasSharedMemory(info$dataId))
    gc()
    ## Check equality
    expect_equal(clusterEvalQ(cl, x)[[1]], 1:10)
    gc()
    expect_equal(x, 1:10)
    gc()
    expect_true(hasSharedMemory(info$dataId))
    ## release it from the child process
    clusterEvalQ(cl,{rm(x);gc()})
    if(getOS() == "windows"){
        expect_true(hasSharedMemory(info$dataId))
    }else{
        expect_false(hasSharedMemory(info$dataId))
    }
    expect_equal(x, 1:10)
    ## release it from the main process
    rm(x)
    gc()
    expect_false(hasSharedMemory(info$dataId))
    stopCluster(cl)
})


test_that("create a shared object from a child process then remove from master first", {
    cl <- makeCluster(1)
    x <- clusterEvalQ(cl, {
        library(SharedObject)
        x <- share(1:10)
        x
    })[[1]]
    info <- sharedObjectProperties(x)
    expect_true(hasSharedMemory(info$dataId))
    gc()
    ## Check equality
    expect_equal(1:10, clusterEvalQ(cl, x)[[1]])
    gc()
    expect_equal(x, 1:10)
    gc()
    expect_true(hasSharedMemory(info$dataId))
    ## release it from the master process
    rm(x)
    gc()
    expect_true(hasSharedMemory(info$dataId))
    expect_equal(clusterEvalQ(cl, x)[[1]], 1:10)
    gc()
    ## release it from the child process
    clusterEvalQ(cl,{rm(x);gc()})
    gc()
    expect_false(hasSharedMemory(info$dataId))
    stopCluster(cl)
})

test_that("create a shared object from a child process then stop cluster", {
    cl <- makeCluster(1)
    x <- clusterEvalQ(cl, {
        library(SharedObject)
        x <- share(1:10)
        x
    })[[1]]
    info <- sharedObjectProperties(x)
    expect_true(hasSharedMemory(info$dataId))
    gc()
    ## Check equality
    expect_equal(clusterEvalQ(cl, x)[[1]], 1:10)
    gc()
    expect_equal(x, 1:10)
    gc()
    expect_true(hasSharedMemory(info$dataId))
    ## Directly stop the cluster
    stopCluster(cl)
    gc()
    if(getOS() == "windows"){
        expect_true(hasSharedMemory(info$dataId))
    }else{
        expect_false(hasSharedMemory(info$dataId))
    }
    expect_equal(x, 1:10)
    ## release it from the main process
    rm(x)
    gc()
    Sys.sleep(1)
    gc()
    expect_false(hasSharedMemory(info$dataId))
})

context("Share list objects")
sharedObjectPkgOptions(minLength = 1)

test_that("list", {
    myData <- list(a = runif(10), b = letters[1:4], d = sample(10,5))
    expect_error({x <- share(myData, mustWork = TRUE)}, NA)
    expect_equal(x, myData)
    expect_equal(is.shared(x), TRUE)
    expect_equal(is.shared(x, depth = 1), list(a=TRUE,b=TRUE,d=TRUE))
})


test_that("data frame", {
    myData <- as.data.frame(matrix(runif(100), 10, 10))
    myData <- cbind(myData,last =letters[seq_len(10)],stringsAsFactors =FALSE)
    expect_error({x <- share(myData, mustWork = TRUE)}, NA)
    expect_equal(x, myData)
    expect_equal(is.shared(x), TRUE)
    expect_equal(as.logical(is.shared(x,depth = 1)), rep(TRUE,11))
})

test_that("type check", {
    data = matrix(0, 2, 2)
    so = share(data)
    expect_equal(is.altrep(so), TRUE)
    expect_equal(is.shared(so), TRUE)
    expect_equal(is.altrep(data), FALSE)
    expect_equal(is.shared(data), FALSE)

    data = as.data.frame(data)
    so = share(data)
    expect_equal(is.altrep(so), FALSE)
    expect_equal(as.logical(is.shared(so, depth = 1)), rep(TRUE,2))
    expect_equal(is.altrep(data), FALSE)
    expect_equal(as.logical(is.shared(data, depth = 1)), rep(FALSE,2))
})


gc()

context("Share basic types")
sharedObjectPkgOptions(minLength = 1)

test_that("basic types",{
    #define NILSXP	     0	  /* nil = NULL */
    value <- NULL
    x <- share(value)
    expect_false(is.shared(x))
    expect_identical(x, value)

    #define SYMSXP	     1	  /* symbols */
    value <- as.symbol("a")
    x <- share(value)
    expect_false(is.shared(x))
    expect_identical(x, value)

    #define LISTSXP	     2	  /* lists of dotted pairs */
    listData <- list(a=runif(10), b=letters[1:10])
    value <- as.pairlist(listData)
    x <- share(value)
    expect_equal(is.shared(x,depth = 1), list(a=TRUE,b=TRUE))
    expect_identical(x, value)

    #define CLOSXP	     3	  /* closures */
    value <- function(x)x
    x <- share(value)
    expect_false(is.shared(x))
    expect_identical(x, value)

    #define ENVSXP	     4	  /* environments */
    value <- as.environment(listData)
    x <- share(value)
    expect_true(is.shared(x))
    expect_identical(x, value)

    #define LANGSXP	     6	  /* language constructs (special lists) */
    value <- call("sin", pi)
    x <- share(value)
    expect_false(is.shared(x))
    expect_identical(x, value)

    #define LGLSXP	    10	  /* logical vectors */
    value <- c(TRUE,FALSE)
    x <- share(value)
    expect_true(is.shared(x))
    expect_identical(x, value)

    #define INTSXP	    13	  /* integer vectors */
    value <- 1L
    x <- share(value)
    expect_true(is.shared(x))
    expect_identical(x, value)

    #define REALSXP	    14	  /* real variables */
    value <- 1.0
    x <- share(value)
    expect_true(is.shared(x))
    expect_identical(x, value)

    #define CPLXSXP	    15	  /* complex variables */
    value <- complex(1)
    x <- share(value)
    expect_true(is.shared(x))
    expect_identical(x, value)

    #define STRSXP	    16	  /* string vectors */
    value <- "a"
    x <- share(value)
    expect_true(is.shared(x))
    expect_identical(x, value)

    #define VECSXP	    19	  /* generic vectors */
    value <- listData
    x <- share(value)
    expect_equal(is.shared(x,depth = 1), list(a=TRUE,b=TRUE))
    expect_identical(x, value)

    #define EXPRSXP	    20	  /* expressions vectors */
    value <- expression(1 + 0:9)
    x <- share(value)
    expect_false(is.shared(x))
    expect_identical(x, value)

    #define RAWSXP      24    /* raw bytes */
    value <- as.raw(1:10)
    x <- share(value)
    expect_true(is.shared(x))
    expect_identical(x, value)

    ## These types will not be supported/tested
    #define PROMSXP	     5	  /* promises: [un]evaluated closure arguments */
    #define SPECIALSXP   7	  /* special forms */
    #define BUILTINSXP   8	  /* builtin non-special forms */
    #define CHARSXP	     9	  /* "scalar" string type (internal only)*/
    #define DOTSXP	    17	  /* dot-dot-dot object */
    #define ANYSXP	    18	  /* make "any" args work.*/
    #define BCODESXP    21    /* byte code */
    #define EXTPTRSXP   22    /* external pointer */
    #define WEAKREFSXP  23    /* weak reference */
    #define S4SXP       25    /* S4, non-vector */
    #define NEWSXP      30    /* fresh node created in new page */
    #define FREESXP     31    /* node released by GC */
    #define FUNSXP      99    /* Closure or Builtin or Special */
})

gc()

context("Share S4 objects")
sharedObjectPkgOptions(minLength = 1)

test_that("atomic type SEXP", {
    .myClass <- setClass("myClass", slots = c(a= "numeric",b="character"), contains = "numeric")
    x <- .myClass(runif(10), a = 1:10, b = letters)
    x1 <- share(x)
    expect_equal(x,x1)
    expect_true(is.shared(x1))
    expect_equal(is.shared(x1, depth = 1), list(.Data= TRUE, a= TRUE, b = TRUE))
    removeClass("myClass")
})

test_that("list type SEXP", {
    .myClass <- setClass("myClass", slots = c(a= "numeric",b="character"), contains = "list")
    x <- .myClass(list(a1 = 1:10, b1 = letters), a = 1:10, b = letters)
    x1 <- share(x)
    expect_equal(x,x1)
    expect_true(is.shared(x1))
    expect_equal(is.shared(x1, depth = 1), list(.Data= TRUE,
                                                a= TRUE, b = TRUE))
    expect_equal(is.shared(x1, depth = 2), list(.Data= list(a1= TRUE,b1=TRUE),
                                                      a= TRUE, b = TRUE))
    removeClass("myClass")
})

test_that("S4 type SEXP", {
    .myClass <- setClass("myClass", slots = c(a= "numeric",b="character"))
    x <- .myClass(a = 1:10, b = letters)
    x1 <- share(x)
    expect_identical(x,x1)
    expect_true(is.shared(x1))
    expect_equal(is.shared(x1, depth = 1), list(a= TRUE, b = TRUE))
    removeClass("myClass")
})
gc()

context("Unshare function")
sharedObjectPkgOptions(minLength = 1)


test_that("unshare Atomic", {
    x <- structure(runif(10), a= 1:10,b=list(a1= 1:2,b1= letters[1:3]))
    x1 <- share(x)
    x2 <- unshare(x1)
    expect_equal(x,x2)
    expect_equal(is.shared(x, depth = Inf, showAttributes = TRUE),
                 is.shared(x2, depth = Inf,showAttributes = TRUE))
})

test_that("unshare list", {
    x <- structure(list(x=runif(10)), a= 1:10,b=list(a1= 1:2,b1= letters[1:3]))
    x1 <- share(x)
    x2 <- unshare(x1)
    expect_equal(x,x2)
    expect_equal(is.shared(x, depth = Inf, showAttributes = TRUE),
                 is.shared(x2, depth = Inf,showAttributes = TRUE))
})

test_that("unshare class", {
    .myClass <- setClass("myClass", slots = c(a= "numeric",b="character", d= "list"),
                         contains = "numeric")
    x <- .myClass(runif(10), a = 1:10, b = letters[1:5], d = list(a1= 1,b1=letters[1:5]))
    x1 <- share(x)
    x2 <- unshare(x1)
    expect_equal(x,x2)
    expect_equal(is.shared(x, depth = Inf, showAttributes = TRUE),
                 is.shared(x2, depth = Inf,showAttributes = TRUE))
    removeClass("myClass")
})

gc()

context("is.shared function")
sharedObjectPkgOptions(minLength = 1)

test_that("is.shared atomic", {
    x <- structure(runif(10), a = 1:10, b = letters[1:5], d=list(a1= 1,b1=letters[1:5]))
    x1 <- share(x)
    expect_equal(is.shared(x1),TRUE)
    expect_equal(is.shared(x1,depth = 1),TRUE)
    expect_equal(is.shared(x1,showAttributes = TRUE),
                 structure(TRUE, aShared = TRUE, bShared = TRUE, dShared = TRUE))
    expect_equal(is.shared(x1,depth = 1,showAttributes = TRUE),
                 is.shared(x1,showAttributes = TRUE))
})

test_that("is.shared list", {
    x <- list(a = 1:10, b = letters[1:5],d=list(a1= 1,b1=letters[1:5]))
    x1 <- share(x)
    expect_true(is.shared(x1))
    expect_equal(is.shared(x1, depth = 1),list(a=TRUE,b=TRUE,d=TRUE))
    expect_equal(is.shared(x1, showAttributes = TRUE),
                 structure(TRUE, namesShared = FALSE))
    expect_equal(is.shared(x1, depth = 1, showAttributes = TRUE),
                 structure(list(a = TRUE, b = TRUE, d = TRUE), namesShared = FALSE))
    expect_equal(is.shared(x1, depth = 2, showAttributes = TRUE),
                 structure(list(a = TRUE, b = TRUE, d = list(a1 = TRUE, b1 = TRUE)),
                           namesShared = FALSE))
})



test_that("is.shared S4", {
    .myClass <- setClass("myClass", slots = c(a= "numeric",b="character", d= "list"),
                         contains = "numeric")
    x <- .myClass(runif(10), a = 1:10, b = letters[1:5], d = list(a1= 1,b1=letters[1:5]))
    x1 <- share(x)
    expect_true(is.shared(x1))
    expect_equal(is.shared(x1, depth = 1), list(.Data= TRUE, a= TRUE, b = TRUE, d= TRUE))
    expect_equal(is.shared(x1, depth = 2),
                 list(.Data= TRUE, a= TRUE, b = TRUE, d= list(a1=TRUE,b1=TRUE)))
    expect_equal(is.shared(x1,showAttributes = TRUE),
                 is.shared(x1))
    expect_equal(is.shared(x1, depth = 1, showAttributes=TRUE),
                 is.shared(x1, depth = 1))
    expect_equal(is.shared(x1, depth = 2, showAttributes=TRUE),
                 is.shared(x1, depth = 2))
    removeClass("myClass")
})
gc()

context("Share S3 objects")
sharedObjectPkgOptions(minLength = 1)

test_that("S3 atomic object", {
    myData <- runif(10)
    class(myData) <- "testClass"

    x = share(myData)
    expect_equal(x, myData)
    expect_equal(is.shared(x, showAttributes = TRUE),
                 structure(TRUE, classShared = FALSE))

    ## Pretending that a vector is a list
    ## if a user provides this kind of confusing object,
    ## We still need to dispatch the share method to the right
    ## function
    myData1 <- structure(myData,class="list")
    x = share(myData1)
    expect_equal(x, myData1)
    expect_equal(is.shared(x, showAttributes = TRUE),
                 structure(TRUE, classShared = FALSE))
})

test_that("S3 list object", {
    myData <- list(a = runif(10), b = letters[1:4], d = sample(10,5))
    class(myData) <- "testClass"

    expect_error({x = share(myData, mustWork = TRUE)}, NA)
    expect_equal(x, myData)
    expect_equal(is.shared(x, depth = 1), list(a=TRUE,b=TRUE,d=TRUE))

    ## Pretending that a list is a matrix
    ## if a user provides this kind of confusing object,
    ## We still need to dispatch the share method to the right
    ## function
    myData1 <- structure(myData, class="matrix")
    x = share(myData1)
    expect_equal(x, myData1)
    expect_equal(is.shared(x, depth = 1), list(a=TRUE,b=TRUE,d=TRUE))
})

test_that("S3 complicated object", {
    x <- lm(mpg~cyl, mtcars)
    x2 <- share(x)
    expect_equal(
        is.shared(x2,depth = 1, showAttributes = TRUE),
        structure(list(coefficients = TRUE, residuals = TRUE, effects = TRUE,
                       rank = TRUE, fitted.values = TRUE, assign = TRUE, qr = TRUE,
                       df.residual = TRUE, xlevels = FALSE, call = FALSE, terms = FALSE,
                       model = TRUE), namesShared = FALSE, classShared = FALSE))

})
gc()

context("Race condition")
library(parallel)

test_that("sharing objects at the same time", {
    cl <- makeCluster(2)
    clusterEvalQ(cl, {
        library(SharedObject)
        sharedObjectPkgOptions(minLength = 1)
    })
    expect_error({
        clusterEvalQ(cl, {
            for(i in 1:10000){
                a <- share(1:10)
            }
        })
    },NA)
    stopCluster(cl)
})

#' Whether an object is an ALTREP object
#'
#' Whether an object is an ALTREP object
#' @param x an R object
#' @return
#' A logical value
is.altrep <- function(x) {
    C_ALTREP(x)
}


isSharableAtomic <- function(x){
    typeof(x) %in% c("raw","logical","integer","double","complex","character")
}
isSEXPList <- function(x){
    is.list(x)&&!is.pairlist(x)
}

## The function return the value in data 1
## if and only if x is a shared object
getAltData1 <- function(x) {
    return(C_getAltData1(x))
}

getAltData2 <- function(x) {
    return(C_getAltData2(x))
}

setAltData1 <- function(x, value) {
    return(C_setAltData1(x, value))
}

setAltData2 <- function(x, value) {
    return(C_setAltData2(x, value))
}

getOSBit <- function(){
    .Machine$sizeof.pointer * 8
}


#' Find path of the shared memory header file
#'
#' This function will return the path of the shared memory header or
#' the flags that are used to compile the package
#' for the developers who want to use C++ level implementation of the
#' `SharedObject` package
#'
#' @param x Character, "PKG_LIBS" or "PKG_CPPFLAGS"
#' @return path to the header or compiler flags
#' @examples
#' SharedObject:::pkgconfig("PKG_LIBS")
#' SharedObject:::pkgconfig("PKG_CPPFLAGS")
pkgconfig <- function(x){
    if(x == "PKG_LIBS"){
        if(.Platform$OS.type=="windows"){
            folder <- sprintf("usrlib/%s", .Platform$r_arch)
            folder <- system.file(folder,
                                  package = "SharedObject", mustWork = FALSE)
        }else{
            folder <- system.file("usrlib",
                                  package = "SharedObject", mustWork = FALSE)
        }
        files <- "SharedObject.a"
        result <- file.path(folder,files)
    }else{
        result <- ""
    }
    cat(result)
    invisible(result)
}

## Get POSIX shared memory files
getSharedFiles <- function(){
    files <- list.files(C_getSharedMemoryPath())
    bits <- as.character(c(32,64))
    res <- list()
    for(i in bits){
        headerId <- paste0("SO_X",i,"_")
        idIndex <- startsWith(files, headerId)
        objectId <- substring(files[idIndex], nchar(headerId)+1)
        res[[as.character(i)]] <- objectId
    }
    res
}

setVerbose<- function(x){
    C_setSharedMemoryPrint(x)
    C_setAltrepPrint(x)
    C_setPackagePrint(x)
}


processArgs <- function(args, literal){
    argNames <- names(args)
    argsSetCommand <- list()
    argsGetCommand <- c()
    if("args"%in%names(args)){
        return(eval(args$args, envir = parent.frame(n=2)))
    }
    for(i in seq_along(args)){
        curArg <- args[[i]]
        if(is.null(argNames)||argNames[i]==""){
            if(literal){
                curArg <- as.character(args[[i]])
            }else{
                curArg <- eval(curArg, envir = parent.frame(n=2))
            }
            argsGetCommand <- c(argsGetCommand, curArg)
        }else{
            if(argNames[i]!="literal"){
                if(is.language(curArg)){
                    curArg <-eval(curArg, envir = parent.frame(n=2))
                }
                argsSetCommand[argNames[i]] <- curArg
            }
        }
    }
    list(argsGetCommand= argsGetCommand, argsSetCommand=argsSetCommand)
}


getOS <- function(){
    sysinf <- Sys.info()
    if (!is.null(sysinf)){
        os <- sysinf['sysname']
        if (os == 'Darwin')
            os <- "osx"
    } else { ## mystery machine
        os <- .Platform$OS.type
        if (grepl("^darwin", R.version$os))
            os <- "osx"
        if (grepl("linux-gnu", R.version$os))
            os <- "linux"
    }
    tolower(os)
}

context("test the path in the package")

## I have no idea why it fails on Bioconductor BBS
## It works perfect fine locally, so I decide to ignore
## it and wait until someone find a way to reproduce it.
# test_that("Package library", {
#     libPath <- SharedObject:::pkgconfig("PKG_LIBS")
#     expect_true(file.exists(libPath))
#     # expect_true(libPath)
# })

test_that("Package flags", {
    libFlags <- capture.output(SharedObject:::pkgconfig("PKG_CPPFLAGS"))
    expect_true(length(libFlags)==0)
})

test_that("shared memory path", {
    memoryPath <- getSharedMemoryPath()
    if(memoryPath!=""){
        expect_true(dir.exists(memoryPath))
    }
})

unshareANY <- function(x){
    if(isS4(x)){
        return(unshareS4(x))
    }else if(isSharableAtomic(x)){
        return(unshareAtomic(x))
    }else if(is.list(x)){
        return(unshareList(x))
    }else if(is.environment(x)){
        return(unshareEnvironment(x))
    }
    x
}

## x must be an atomic object
unshareAtomic <- function(x) {
    oldAttrs <- attributes(x)
    newAttrs <- unshare(oldAttrs)
    ## If the object is a shared object.
    if(C_isShared(x)){
        if(typeof(x)!="character"){
            unshareFunc <- C_unshare
        }else{
            unshareFunc <- C_unshareString
        }
        result<- unshareFunc(x, as.pairlist(newAttrs))
    }else{
        result <- x
        if(!C_isSameObject(newAttrs,oldAttrs)){
            attributes(result) <- newAttrs
        }
    }
    result
}

unshareList <- function(x) {
    doList(unshare,unshare,x)
}

unshareS4 <- function(x){
    doS4(unshare,x)
}
unshareEnvironment <- function(x){
    doEnvironment(unshare,unshare,x)
}

#' @export
setMethod("unshare", signature(x = "ANY"), unshareANY)

#' List all shared Objects
#'
#' @param start the start value of the ID. The default is `NULL`. See details.
#' @param end the end value of the ID. The default is `NULL`. See details.
#'
#' @details
#' The parameter `start` and `end` specify the range of the ID. If not specified, all
#' IDs will be listed.
#'
#' On Ubuntu or some other linux systems, the shared objects
#' can be found in the folder `/dev/shm`. The function can find all shared objects
#' if the folder exists.
#'
#' On Windows, since there is no easy way to find all shared objects.
#' the function will guess the range of the shared object IDs and search
#' for all IDs within the range. Therefore, if there are too many shared
#' objects(over 4 billions) ,the object id can be out of the searching range
#' and the result may not be complete.
#' Furthermore, there will be no named shared memory in the returned list.
#'
#' Note that the size in the return value is the true memory size
#' that is reserved by the shared object, so it might be larger
#' than the object size.
#'
#' @examples
#' x <- share(runif(10))
#' ## Automatically determine the search range
#' listSharedObjects()
#'
#' ## specify the search range
#' listSharedObjects(start = 10, end = 20)
#'
#' ## Search from 0 to 20
#' listSharedObjects(20)
#' @seealso \code{\link{getLastIndex}}, \code{\link{allocateSharedMemory}},
#' \code{\link{mapSharedMemory}}, \code{\link{unmapSharedMemory}},
#' \code{\link{freeSharedMemory}}, \code{\link{hasSharedMemory}}, \code{\link{getSharedMemorySize}}
#' @return A data.frame object with shared object id and size
#' @export
listSharedObjects <- function(end = NULL,start = NULL) {
    shared_memory_path <- C_getSharedMemoryPath()
    if(shared_memory_path!="" && file.exists(shared_memory_path)){
        usedIds <- getSharedFiles()
        usedIds <- usedIds[[as.character(getOSBit())]]
        if(!is.null(start))
            usedIds <- usedIds[usedIds>=start]
        if(!is.null(end))
            usedIds <- usedIds[usedIds<=end]
    }else{
        if(is.null(start))
            start <- 0
        if(is.null(end))
            end <- getLastIndex()
        if(end < start)
            ids <- c()
        else
            ids <- seq_len(end - start + 1) + start - 1
        usedIds <- ids[vapply(ids, hasSharedMemory, logical(1))]
    }
    memorySize <- unlist(lapply(usedIds, function(id)
        {
        tryCatch(
            getSharedMemorySize(id),
            error = function(e) -1
        )
    }))
    idx <- (memorySize != -1)
    data.frame(Id = usedIds[idx], size = memorySize[idx], row.names = NULL)
}



##########################################
## Functions to create shared memory
##########################################
#' Functions to manipulate shared memory
#'
#' These functions are designed for package developers only, they can
#' allocate, open, close and destroy shared memory without touching
#' C++ code. Normal users should not use these functions unless
#' dealing with memory leaking
#'
#' @param name,id The name/id of the shared memory. Can be a vector
#' @param size The size of the shared memory that you want to allocate. Can be a vector.
#' If the argument `name` presents, The length of `size` must match the length of `name`.
#'
#' @details
#' \strong{Quick explanation}
#'
#' `getLastIndex`: the ID of the last created shared memory.
#'
#' `allocateSharedMemory`: allocate a shared memory of a given size,
#' the memory ID is returned by the function
#'
#' `mapSharedMemory`: map the shared memory to the current process memory space
#'
#' `unmapSharedMemory`: unmap the shared memory(without destroying it)
#'
#' `freeSharedMemory`: free the shared memory. This function will only unmap the
#' shared memory on Windows.
#'
#' `hasSharedMemory`: whether the memory exist?
#'
#' `getSharedMemorySize`: get the actual size of the shared memory, it may be larger than the
#' size you required.
#'
#' \strong{Details}
#'
#' A complete lifecycle of a shared memory involves four steps:
#' allocating, mapping, unmapping and freeing the shared memory.
#'
#' The shared memory can be created by `allocateSharedMemory`.
#' The function `allocateSharedMemory` will return the ID of the shared memory.
#' After creating the shared memory, it can be mapped to the current process by
#' `mapSharedMemory`. The return value is an external pointer to the shared memory.
#' Once the shared memory is no longer needed, it can be unmapped and destroyed by
#' `unmapSharedMemory` and `freeSharedMemory` respectively.
#'
#'
#' @return
#' `getLastIndex`: An interger ID served as a hint of the last created shared memory ID.
#'
#' `allocateSharedMemory`: character ID(s) that can be used to find the shared memory
#'
#' `mapSharedMemory`: External pointer(s) to the shared memory
#'
#' `unmapSharedMemory`: No return value
#'
#' `freeSharedMemory`: No return value
#'
#' `hasSharedMemory`: Logical value(s) indicating whether the shared memory exist
#'
#' `getSharedMemorySize`: A numeric value
#' @seealso \code{\link{listSharedObjects}}
#' @rdname developer-API
#' @examples
#' size <- 10L
#' ## unnamed shared memory
#' id <- allocateSharedMemory(size)
#' hasSharedMemory(id)
#' ptr <- mapSharedMemory(id)
#' ptr
#' getSharedMemorySize(id)
#' unmapSharedMemory(id)
#' freeSharedMemory(id)
#' hasSharedMemory(id)
#'
#' ## named shared memory
#' name <- "SharedObjectExample"
#' if(!hasSharedMemory(name)){
#'     allocateSharedMemory(size, name = name)
#'     hasSharedMemory(name)
#'     ptr <- mapSharedMemory(name)
#'     ptr
#'     getSharedMemorySize(name)
#'     unmapSharedMemory(name)
#'     freeSharedMemory(name)
#'     hasSharedMemory(name)
#' }
#' @export
getLastIndex <- function(){
    C_getLastIndex()
}

#' @rdname developer-API
#' @export
allocateSharedMemory <- function(size, name = ""){
    if(length(size)>1&&name==""){
        name <- rep(name, length(size))
    }
    stopifnot(length(size)==length(name))
    if(length(size)>1){
        lapply(seq_along(size),
               function(i)allocateSharedMemory(size[i], name[i]))
    }else{
        C_allocateSharedMemory(size, name)
    }
}
#' @rdname developer-API
#' @export
mapSharedMemory <- function(id){
    if(length(id)>1){
        lapply(id, mapSharedMemory)
    }else{
        id <- as.character(id)
        C_mapSharedMemory(id)
    }
}
#' @rdname developer-API
#' @export
unmapSharedMemory <- function(id){
    if(length(id)>1){
        lapply(id, unmapSharedMemory)
    }else{
        id <- as.character(id)
        C_unmapSharedMemory(id)
    }
    invisible()
}
#' @rdname developer-API
#' @export
freeSharedMemory <- function(id){
    if(length(id)>1){
        lapply(id, freeSharedMemory)
    }else{
        id <- as.character(id)
        C_freeSharedMemory(id)
    }
    invisible()
}
#' @rdname developer-API
#' @export
hasSharedMemory <- function(id){
    if(length(id)>1){
        lapply(id, hasSharedMemory)
    }else{
        id <- as.character(id)
        C_hasSharedMemory(id)
    }
}
#' @rdname developer-API
#' @export
getSharedMemorySize <- function(id){
    if(length(id)>1){
        lapply(id, getSharedMemorySize)
    }else{
        id <- as.character(id)
        C_getSharedMemorySize(id)
    }
}
#' @rdname developer-API
#' @export
initialSharedObjectPackageData <- function(){
    C_initialPkgData()
}
#' @rdname developer-API
#' @export
releaseSharedObjectPackageData <- function(){
    C_releasePkgData()
}

context("developer APIs")
sharedObjectPkgOptions(minLength = 1)

test_that("Testing memory tools", {
    x <- share(1:10)
    info <- sharedObjectProperties(x)
    check1 <- hasSharedMemory(info$dataId)
    expect_true(check1)
    check2 <- getSharedMemorySize(info$dataId)
    expect_true(check2>=info$totalSize)
})

test_that("Create memory by ID", {
    size <- 10
    id <- allocateSharedMemory(size)
    expect_true(is.character(id))
    result <- hasSharedMemory(id)
    expect_true(result)
    ptr <- mapSharedMemory(id)
    expect_true(is(ptr,"externalptr"))
    result <- getSharedMemorySize(id)
    expect_true(result>=size)
    unmapSharedMemory(id)
    result <- hasSharedMemory(id)
    if(getOS()=="windows"){
        expect_false(result)
    }else{
        expect_true(result)
    }
    freeSharedMemory(id)
    result <- hasSharedMemory(id)
    expect_false(result)
})

test_that("Create memory by name", {
    name <- "SharedObjectPackageTest"
    size <- 10
    noMemory <- TRUE
    if(hasSharedMemory(name)){
        freeSharedMemory(name)
    }
    expect_true(!hasSharedMemory(name))
    if(!hasSharedMemory(name)){
        expect_equal(allocateSharedMemory(size, name),
                     name)
        ## Repeat allocating the same memory
        expect_error(allocateSharedMemory(size, name), NA)
        expect_error(allocateSharedMemory(1000000, name))
        ## Check memory existence
        result <- hasSharedMemory(name)
        expect_true(result)
        ptr <- mapSharedMemory(name)
        expect_true(is(ptr,"externalptr"))
        result <- getSharedMemorySize(name)
        expect_true(result>=size)
        unmapSharedMemory(name)
        result <- hasSharedMemory(name)
        if(getOS()=="windows"){
            expect_false(result)
        }else{
            expect_true(result)
        }
        freeSharedMemory(name)
        result <- hasSharedMemory(name)
        expect_false(result)
    }
})




test_that("Create memory by ID without unmap", {
    size <- 10
    id <- allocateSharedMemory(size)
    expect_true(is.character(id))
    ptr <- mapSharedMemory(id)
    expect_true(is(ptr,"externalptr"))
    freeSharedMemory(id)
    result <- hasSharedMemory(id)
    expect_false(result)
})

test_that("Create memory by name without unmap", {
    name <- "SharedObjectPackageTest"
    size <- 10
    if(hasSharedMemory(name)){
        freeSharedMemory(name)
    }
    expect_true(!hasSharedMemory(name))
    if(!hasSharedMemory(name)){
        allocateSharedMemory(size, name)
        ptr <- mapSharedMemory(name)
        expect_true(is(ptr,"externalptr"))
        freeSharedMemory(name)
        result <- hasSharedMemory(name)
        expect_false(result)
    }
})

if(getOS() == "linux"){
    test_that("listSharedObjects", {
        gc()
        expect_equal(nrow(listSharedObjects()), 0)
        x <- lm(mpg~cyl, mtcars)
        x2 <- share(x)
        expect_true(nrow(listSharedObjects())>0)
        rm(list = "x2")
        gc()
        expect_equal(nrow(listSharedObjects()), 0)
    })
    test_that("list named shared object", {
        name <- "SharedObjectPackageTest2"
        size <- 10
        if(hasSharedMemory(name)){
            freeSharedMemory(name)
        }
        expect_true(!hasSharedMemory(name))
        if(!hasSharedMemory(name)){
            allocateSharedMemory(size,name)
            expect_equal(nrow(listSharedObjects()),1)
            freeSharedMemory(name)
            expect_equal(nrow(listSharedObjects()),0)
        }
    })
}

## we want to dispatch the share function based on the SEXP type, not the
## `class` attribute, so the S4 dispatch system does not fit
## our requirement, we create our own dispatching rule:
## 1. If the SEXP is an S4 object, dispatch to shareS4 method
## 2. If the SEXP is an atomic object,
## dispatch to shareAtomic function
## 3. If SEXP is a list, dispatch to shareList
## 4. If SEXP is an environment, dispatch to shareEnvironment
## 4. Otherwise, for any unknown object, throw an error or ignore it
shareANY <- function(x, ..., copyOnWrite, sharedSubset, sharedCopy,
                     sharedAttributes, mustWork, minLength){
    allArgs <- list(x = x, ...)
    if(!missing(copyOnWrite))
        allArgs[["copyOnWrite"]] <- copyOnWrite
    if(!missing(sharedSubset))
        allArgs[["sharedSubset"]] <- sharedSubset
    if(!missing(sharedCopy))
        allArgs[["sharedCopy"]] <- sharedCopy
    if(!missing(sharedAttributes))
        allArgs[["sharedAttributes"]] <- sharedAttributes
    if(!missing(mustWork))
        allArgs[["mustWork"]] <- mustWork
    if(!missing(minLength))
        allArgs[["minLength"]] <- minLength

    if(isS4(x)){
        return(do.call(shareS4, allArgs, quote = TRUE))
    }else if(isSharableAtomic(x)){
        return(do.call(shareAtomic, allArgs, quote = TRUE))
    }else if(is.list(x)){
        return(do.call(shareList, allArgs, quote = TRUE))
    }else if(is.environment(x)){
        return(do.call(shareEnvironment, allArgs, quote = TRUE))
    }
    do.call(promptError, allArgs, quote = TRUE)
}


## x must be an atomic object
shareAtomic <- function(x,...) {
    options <- completeOptions(...)
    oldAttrs <- attributes(x)
    if(options$sharedAttributes){
        newAttrs <- doAttributes(tryShare, oldAttrs,...)
    }else{
        newAttrs <- oldAttrs
    }
    if(C_xlength(x)>=options$minLength){
        if(typeof(x)!="character"){
            result <- C_createSharedObjectFromSource(
                x=x,
                copyOnWrite=options$copyOnWrite,
                sharedSubset=options$sharedSubset,
                sharedCopy=options$sharedCopy,
                attributes=as.pairlist(newAttrs)
            )
        }else{
            result <- C_createSharedStringFromSource(
                x=x,
                copyOnWrite=options$copyOnWrite,
                attributes=as.pairlist(newAttrs)
            )
        }
        # Set the object property for the shared object
        C_setObject(result, C_getObject(x))
    }else{
        result <- x
        if(!C_isSameObject(newAttrs,oldAttrs)){
            attributes(result) <- newAttrs
        }
    }
    result
}

shareList <- function(x,...) {
    doList(share,tryShare,x,...)
}


shareS4 <- function(x,...){
    doS4(tryShare,x,...)
}
shareEnvironment <- function(x,...){
    doEnvironment(share,tryShare,x,...)
}

promptError <- function(x, ...) {
    options <- completeOptions(...)
    mustWork <- options$mustWork
    if(is.null(x))
        return(x)
    if (!mustWork)
        return(x)
    stop(
        "The object of the class <",
        paste0(class(x),collapse = ", "),
        "> cannot be shared.\n",
        "To suppress this error and return the same object, \n",
        "provide `mustWork = FALSE` as a function argument\n",
        "or change its default value in the package settings\n"
    )
}


#' @rdname share
#' @export
setMethod("share", signature(x = "ANY"), shareANY)

## We use tryShare internally
tryShare <- function(x, ...) {
    options <- list(x = x, ...)
    options["mustWork"] <- FALSE
    do.call(share, options, quote = TRUE)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

C_getDataInfoTemplate <- function() {
    .Call(`_SharedObject_C_getDataInfoTemplate`)
}

C_getStringDataInfoTemplate <- function() {
    .Call(`_SharedObject_C_getStringDataInfoTemplate`)
}

C_createEmptySharedObject <- function(type, length, copyOnWrite, sharedSubset, sharedCopy, attributes) {
    .Call(`_SharedObject_C_createEmptySharedObject`, type, length, copyOnWrite, sharedSubset, sharedCopy, attributes)
}

C_createSharedObjectFromSource <- function(x, copyOnWrite, sharedSubset, sharedCopy, attributes) {
    .Call(`_SharedObject_C_createSharedObjectFromSource`, x, copyOnWrite, sharedSubset, sharedCopy, attributes)
}

C_createSharedStringFromSource <- function(x, copyOnWrite, attributes) {
    .Call(`_SharedObject_C_createSharedStringFromSource`, x, copyOnWrite, attributes)
}

C_readSharedObject <- function(dataInfo) {
    .Call(`_SharedObject_C_readSharedObject`, dataInfo)
}

C_unshare <- function(x, attributes) {
    .Call(`_SharedObject_C_unshare`, x, attributes)
}

C_unshareString <- function(x, attributes) {
    .Call(`_SharedObject_C_unshareString`, x, attributes)
}

C_setSharedObjectOwership <- function(x, ownData) {
    invisible(.Call(`_SharedObject_C_setSharedObjectOwership`, x, ownData))
}

C_getSharedObjectOwership <- function(x) {
    .Call(`_SharedObject_C_getSharedObjectOwership`, x)
}

C_xlength <- function(x) {
    .Call(`_SharedObject_C_xlength`, x)
}

C_ALTREP <- function(x) {
    .Call(`_SharedObject_C_ALTREP`, x)
}

C_getAltData1 <- function(x) {
    .Call(`_SharedObject_C_getAltData1`, x)
}

C_getAltData2 <- function(x) {
    .Call(`_SharedObject_C_getAltData2`, x)
}

C_isShared <- function(x) {
    .Call(`_SharedObject_C_isShared`, x)
}

C_setAltData1 <- function(x, data) {
    invisible(.Call(`_SharedObject_C_setAltData1`, x, data))
}

C_setAltData2 <- function(x, data) {
    invisible(.Call(`_SharedObject_C_setAltData2`, x, data))
}

C_getObject <- function(x) {
    .Call(`_SharedObject_C_getObject`, x)
}

C_setObject <- function(x, i) {
    invisible(.Call(`_SharedObject_C_setObject`, x, i))
}

C_ISS4 <- function(x) {
    .Call(`_SharedObject_C_ISS4`, x)
}

C_SETS4 <- function(x) {
    invisible(.Call(`_SharedObject_C_SETS4`, x))
}

C_UNSETS4 <- function(x) {
    invisible(.Call(`_SharedObject_C_UNSETS4`, x))
}

C_isSameObject <- function(x, y) {
    .Call(`_SharedObject_C_isSameObject`, x, y)
}

C_getDataTypeId <- function(type) {
    .Call(`_SharedObject_C_getDataTypeId`, type)
}

C_initialPkgData <- function() {
    invisible(.Call(`_SharedObject_C_initialPkgData`))
}

C_releasePkgData <- function() {
    invisible(.Call(`_SharedObject_C_releasePkgData`))
}

C_getLastIndex <- function() {
    .Call(`_SharedObject_C_getLastIndex`)
}

C_allocateSharedMemory <- function(size_in_byte, name = "") {
    .Call(`_SharedObject_C_allocateSharedMemory`, size_in_byte, name)
}

C_mapSharedMemory <- function(id) {
    .Call(`_SharedObject_C_mapSharedMemory`, id)
}

C_unmapSharedMemory <- function(id) {
    invisible(.Call(`_SharedObject_C_unmapSharedMemory`, id))
}

C_freeSharedMemory <- function(id) {
    invisible(.Call(`_SharedObject_C_freeSharedMemory`, id))
}

C_hasSharedMemory <- function(id) {
    .Call(`_SharedObject_C_hasSharedMemory`, id)
}

C_getSharedMemorySize <- function(id) {
    .Call(`_SharedObject_C_getSharedMemorySize`, id)
}

C_getSharedMemoryPath <- function() {
    .Call(`_SharedObject_C_getSharedMemoryPath`)
}

createSharedObjectFromSource <- function(x, copyOnWrite, sharedSubset, sharedCopy, attributes) {
    .Call(`_SharedObject_createSharedObjectFromSource`, x, copyOnWrite, sharedSubset, sharedCopy, attributes)
}

createSharedStringFromSource <- function(x, copyOnWrite, attributes) {
    .Call(`_SharedObject_createSharedStringFromSource`, x, copyOnWrite, attributes)
}

getSharedObjectList <- function() {
    .Call(`_SharedObject_getSharedObjectList`)
}

C_setSharedMemoryPrint <- function(x) {
    invisible(.Call(`_SharedObject_C_setSharedMemoryPrint`, x))
}

C_setAltrepPrint <- function(x) {
    invisible(.Call(`_SharedObject_C_setAltrepPrint`, x))
}

C_setPackagePrint <- function(x) {
    invisible(.Call(`_SharedObject_C_setPackagePrint`, x))
}

getSharedMemoryPath <- function() {
    .Call(`_SharedObject_getSharedMemoryPath`)
}

## recursive: Whether to show the details of all elements
## of the object x which are not directly associated with x
## showAttributes: whether to show the information of the attributes of x
isSharedSEXP <- function(x, showAttributes = FALSE, depth = 1){
    result <- FALSE
    if (is.altrep(x)) {
        result <- C_isShared(x)
    }
    attrs <- attributes(x)
    if(showAttributes&&!is.null(attrs)&& depth>0){
        attrsShared <- rep(list(NA),length(attrs))
        names(attrsShared) <- paste0(names(attrs), "Shared")
        for(i in seq_along(attrs)){
            attrsShared[[i]] <- is.shared(attrs[[i]], showAttributes = FALSE, depth = depth - 1)
        }
        attributes(result) <- attrsShared
    }
    result
}

isSharedANY <- function(x,...,depth,showAttributes){
    ## If the object is neither an S4 object or a list
    ## Just check if the SEXP is a shared altrep object
    result <- isSharedSEXP(x,showAttributes=showAttributes)
    result
}
isSharedList <- function(x,...,depth,showAttributes){
    result <- lapply(x, function(x,...)is.shared(x,...),
                     ...,depth=depth-1L, showAttributes=FALSE)
    if(depth<=0){
        result <- any(unlist(result))
    }
    attrs <- attributes(x)
    if(showAttributes&&!is.null(attrs)){
        attrsShared <- is.shared(attrs, showAttributes = FALSE, depth = 1)
        names(attrsShared) <- paste0(names(attrs), "Shared")
        if(depth>0){
            if("names"%in%names(attrs)){
                attrsShared["names"] <-attrs["names"]
            }
        }
        attributes(result) <- attrsShared
    }
    result
}

isSharedS4 <- function(x,...,depth,showAttributes){
    slots <- slotNames(x)
    result <- vector("list",length(slots))
    for(i in seq_along(slots)){
        result[[i]] <- is.shared(slot(x, slots[i]),...,depth=depth-1L, showAttributes = FALSE)
    }
    names(result) <- slots
    if(".Data" %in% names(result)){
        if(isSharableAtomic(x)){
            result[[".Data"]] <- isSharedSEXP(x,showAttributes = FALSE)
        }
        if(isSEXPList(x)&&is.list(result[[".Data"]])){
            names(result[[".Data"]]) <- names(x)
        }
    }
    ## remove the empty slot
    # result <- result[unlist(lapply(result, function(x) length(x) != 0))]
    if(depth<=0){
        result <- any(unlist(result))
    }
    result
}



#' @rdname is.shared
#' @export
setMethod("is.shared", "ANY", function(x,...,depth,showAttributes){
    if(isS4(x)){
        return(isSharedS4(x,...,depth=depth,showAttributes=showAttributes))
    }
    if(is.list(x)||is.environment(x)){
        return(isSharedList(x,...,depth=depth,showAttributes=showAttributes))
    }
    isSharedANY(x,...,depth=depth,showAttributes=showAttributes)
})

#############################
## constructor for a shared object
#############################
#' Create a shared object
#'
#' This function will create a shared object for the object `x`.
#' The behavior of the shared object is exactly the same as `x`,
#' but the data of the shared object is allocated in the shared
#' memory space. Therefore, a shared object can be easily exported to the other
#' R workers without duplicating the data, which can reduce the memory consumption
#' and the overhead of data transmission.
#'
#' @param x An R object that will be shared, see details.
#' @param copyOnWrite,sharedSubset,sharedCopy The parameters
#' controlling the behavior of a shared object, see details.
#' @param mustWork Whether to throw an error if `x` is not sharable
#' (e.g. x is a function).
#' This parameter has no effect on the object's attributes and S4 object.
#' @param sharedAttributes Whether to share the attributes of the object `x`
#' (default `TRUE`). Note that attribute `class` and `names` will never be shared.
#' @param minLength The minimum length of a shared object(default `3`).
#' If `length(x)` is smaller than the minimum length, it would not be shared.
#' This parameter can be used to reduce the memory fragmentation.
#' @param ... For generalization purpose.
#'
#' @aliases share,vector-method share,matrix-method
#' share,data.frame-method share,list-method
#'
#' @return A shared object
#' @details
#' The function returns a shared object corresponding to the argument `x` if it
#' is sharable. There should be no different between `x` and the return value except
#' that the latter one is shared. The attributes of `x` will also be shared if possible.
#'
#' **Supported types**
#'
#' For the basic R type, the function supports `raw`, `logical`,`integer`,
#'  `double`, `complex`. `character` can be shared, but sharing a `character`
#' is beneficial only when there are a lot repetitions in the
#' elements of the vector. Due to the complicated structure of the character
#' vector, you are not allowed to set the value of a shared
#' character vector to a value which haven't presented in the vector before.
#' It is recommended to treat a shared character vector as read-only.
#'
#' For the container, the function supports `list`, `pairlist`
#' and `environment`. Note that sharing a container is equivalent
#' to share all elements in the container, the container itself
#' will not be shared.
#'
#' The function `share` is an S4 generic. The default share method works for
#' most S3/S4 objects. Therefore, there is no need to define a S4 share method
#' for each S3/S4 class unless the S3/S4 class has a special implementation
#' (e.g. on-disk data).
#' The default method will share all slots the object contains and the object itself
#' if possible. No error will be given if any of these objects are not
#' sharable and they will be kept unchanged.
#'
#'
#' **Behavior control**
#'
#' The behavior of a shared object can be controlled through three parameters:
#' `copyOnWrite`, `sharedSubset` and `sharedCopy`.
#'
#' `copyOnWrite` determines Whether a shared object needs to be duplicated when the
#' data of the shared object is changed. The default value is `TRUE`,
#' but can be altered by passing `copyOnWrite = FALSE` to the function.
#' This parameter can be used to let workers directly write the result back to
#' a shared object.
#'
#' Please note that the no-copy-on-write feature is not fully supported by R. When
#' `copyOnWrite` is `FALSE`, a shared object might not behaves as one expects.
#' Please refer to the example code to see the exceptions.
#'
#' `sharedSubset` determines whether the subset of a shared object is still a shared object.
#'  The default value is `FALSE`, and can be changed by passing `sharedSubset = TRUE`
#'  to the function
#'
#'  At the time of writing, The shared subset feature will
#'  cause an unnecessary memory duplication in R studio. Therefore, for
#'  the performance consideration, it is recommended to keep the feature
#'  off in R studio.
#'
#' `sharedCopy` determines whether the object is still a shared object after the
#' duplication. Note that it must be used with `copyOnWrite = TRUE`. Otherwise,
#' the shared object will never be duplicated. The default value is `FALSE`.
#'
#'
#' @examples
#' ## For vector
#' x <- runif(10)
#' so <- share(x)
#' x
#' so
#'
#' ## For matrix
#' x <- matrix(runif(10), 2, 5)
#' so <- share(x)
#' x
#' so
#'
#' ## For data frame
#' x <- as.data.frame(matrix(runif(10), 2, 5))
#' so <- share(x)
#' x
#' so
#'
#' ## export the object
#' library(parallel)
#' cl <- makeCluster(1)
#' clusterExport(cl, "so")
#' ## check the exported object in the other process
#' clusterEvalQ(cl, so)
#'
#' ## close the connection
#' stopCluster(cl)
#'
#' ## Copy on write
#' x <- runif(10)
#' so1 <- share(x, copyOnWrite = TRUE)
#' so2 <- so1
#' so2[1] <- 10
#' ## so1 is unchanged since copy-on-write feature is on.
#' so1
#' so2
#'
#' ## No copy on write
#' so1 <- share(x, copyOnWrite = FALSE)
#' so2 <- so1
#' so2[1] <- 10
#' #so1 is changed
#' so1
#' so2
#'
#' ## Flaw of no-copy-on-write
#' ## The following code changes the value of so1,
#' ## highly unexpected! Please use with caution!
#' -so1
#' so1
#' ## The reason is that the minus function tries to
#' ## duplicate so1 object, but the duplication function
#' ## will return so1 itself, so the values in so1 get changed.
#'
#' @rdname share
#' @export
setGeneric("share", signature="x", function(x,
                                            ...) {
    standardGeneric("share")
})
#' Unshare a shared object
#'
#' Unshare a shared object. There will be no effect if the
#' object is not shared.
#'
#' @param x a shared object, or an object that contains a shared object.
#' @return An unshared object
#' @aliases unshare,ANY-method unshare,vector-method unshare,list-method
#' @examples
#' x1 <- share(1:10)
#' x2 <- unshare(x1)
#' is.shared(x1)
#' is.shared(x2)
#' @export
setGeneric("unshare", signature="x", function(x){
    standardGeneric("unshare")
})


#' Test whether an object is shared
#'
#' @param x An R object
#' @param depth Whether to recursively check the element of `x`. This parameter
#' only works for container objects(e.g. `list` and `environment`), see details.
#' @param showAttributes Whether to check the attributes of `x`, default `FALSE`.
#' @param ... For generalization purpose only
#' @details
#' When `depth=0`, the `is.shared` function return a single logical value indicating
#' whether `x` is shared or contains any shared objects. When `depth>0` and `x` is a
#' container(e.g. `list`), the function will recursively check each element of `x` and
#' return a list with each elements corresponding to the elements in `x`.
#' The `depth` parameter determines the depth of the checking procedure.
#'
#' if `showAttributes = TRUE`, the attributes of the object will also be checked. The
#' check result is returned as attributes of the return value by appending `Shared` to the
#' end of the original attribute names. Note that `showAttributes` has
#' no effect on an S4 object for the attributes of an S4 object are used to store the
#' slots and should not be treated as the attributes of an object.
#'
#' @return a single logical value or a list.
#' @examples
#' x1 <- share(1:10)
#' is.shared(x1)
#'
#' x2 <- share(list(a=1:10, b = list(d = letters, e = runif(10))))
#' is.shared(x2, depth=0)
#' is.shared(x2, depth=0, showAttributes = TRUE)
#' is.shared(x2, depth=1)
#' is.shared(x2, depth=2)
#' @rdname is.shared
#' @export
setGeneric("is.shared", signature="x",
           function(x, ..., depth = 0, showAttributes  = FALSE) {
               standardGeneric("is.shared")
           })




#' Get/Set the properties of a shared object.
#'
#' @param x A shared object
#' @param literal Whether the parameters in `...` are always treated as characters.
#' @param ... The name of the property(s), it can be either symbols or characters.
#' if the argument is missing, it means getting all properties. See examples.
#' @details
#' For numeric objects, the properties are `dataId`, `length`,
#' `totalSize`, `dataType`, `ownData`, `copyOnWrite`, `sharedSubset`,
#' `sharedCopy`.
#'
#' For character objects, the properties are
#' `length`, `unitSize`,`totalSize`,`dataType`,`uniqueChar`,`copyOnWrite`.
#'
#' Note that only `copyOnWrite`, `sharedSubset` and `sharedCopy` are mutable.
#' The other attributes are read-only.
#' @return
#' get: The property(s) of a shared object
#'
#' set: The old property(s)
#' @examples
#' ## For numeric objects
#' x1 <- share(1:10)
#'
#' ## Get attributes
#' sharedObjectProperties(x1)
#' 
#' ## Get a specific property
#' ## Both symbol and character give the same result
#' sharedObjectProperties(x1, copyOnWrite)
#' sharedObjectProperties(x1, "copyOnWrite")
#' 
#' ## Shortcut for obtaining copy-on-write property
#' getCopyOnWrite(x1)
#' 
#' ## Use a character vector
#' props <- c("copyOnWrite","sharedSubset")
#' sharedObjectProperties(x1, props, literal = FALSE)
#'
#' ## Set attributes
#' sharedObjectProperties(x1, copyOnWrite = FALSE)
#' setCopyOnWrite(x1, FALSE)
#'
#' ## For character objects
#' x2 <- share(letters)
#' sharedObjectProperties(x2)
#' @rdname sharedObjectProperties
#' @export
setGeneric("sharedObjectProperties", function(x, ..., literal = TRUE) {
    standardGeneric("sharedObjectProperties")
})

getAtomicSharedObjectProperties <-  function(x, property) {
    if (!isSharedSEXP(x)) {
        return(NULL)
    }
    if(!is.character(x)){
        dataInfoTemplate <- C_getDataInfoTemplate()
    }else{
        dataInfoTemplate <- C_getStringDataInfoTemplate()
    }
    if (missing(property) || is.null(property)) {
        property <- names(dataInfoTemplate)
    }else{
        property <-
            property[property %in% names(dataInfoTemplate)]
    }
    info <- C_getAltData2(x)
    if("ownData"%in% names(dataInfoTemplate) && "ownData"%in% property){
        info[["ownData"]] <- C_getSharedObjectOwership(x)
    }
    if (length(property) == 1) {
        return(info[[property]])
    } else{
        return(info[property])
    }
}

setAtomicSharedObjectProperties <- function(x, property, value) {
    if (!isSharedSEXP(x)) {
        return(invisible())
    }
    dataInfo <- C_getAltData2(x)
    if (any(!property %in% names(dataInfo))) {
        stop("The property '",
             paste0(property[!property %in% names(dataInfo)], collapse =
                        ", "),
             "' is not found")
    }
    value <- rep_len(value, length(property))
    oldInfo <- dataInfo[property]
    for (i in seq_along(property)) {
        dataInfo[[property[i]]] <- as(value[i], class(dataInfo[[property[i]]]))
    }
    setAltData2(x, dataInfo)
    if("ownData" %in% names(dataInfo)){
        if("ownData" %in% property)
            C_setSharedObjectOwership(x, dataInfo[["ownData"]])
    }

    if(length(property)==1){
        invisible(oldInfo[[1]])
    }else{
        invisible(oldInfo)
    }
}

getOrSetAtomicSharedObjectProperties <- function(x, args){
    if(length(args$argsGetCommand)!=0&&
       length(args$argsSetCommand)!=0){
        stop("You cannot get and set the properties of a shared object at the same time")
    }
    if(length(args$argsSetCommand)!=0){
        setAtomicSharedObjectProperties(x, names(args$argsSetCommand),
                                   unlist(args$argsSetCommand))
    }else{
        getAtomicSharedObjectProperties(x, args$argsGetCommand)
    }
}

#' @rdname sharedObjectProperties
#' @export
setMethod("sharedObjectProperties", signature(x = "ANY"),
          function(x,...,literal){
              sysCall <- as.list(sys.call())[c(-1,-2)]
              args <- processArgs(sysCall,literal)
              getOrSetAtomicSharedObjectProperties(x, args)
          })

#' @rdname sharedObjectProperties
#' @export
setMethod("sharedObjectProperties", signature(x = "list"),
          function(x,...,literal) {
              sysCall <- as.list(sys.call())[c(-1,-2)]
              args <- processArgs(sysCall,literal)
              if(is.atomic(x)){
                  res <- getOrSetAtomicSharedObjectProperties(x, args)
              }else{
                  res <- vector("list", length(x))
                  names(res) <- names(x)
                  for(i in seq_along(x)){
                      res[[i]] <- sharedObjectProperties(x[[i]], args=args)
                  }
              }
              res
          })





#' @param value The value of the property
#' @rdname sharedObjectProperties
#' @export
getCopyOnWrite <- function(x) {
    sharedObjectProperties(x, "copyOnWrite")
}
#' @rdname sharedObjectProperties
#' @export
getSharedSubset <- function(x) {
    sharedObjectProperties(x, "sharedSubset")
}
#' @rdname sharedObjectProperties
#' @export
getSharedCopy <- function(x) {
    sharedObjectProperties(x, "sharedCopy")
}

#' @rdname sharedObjectProperties
#' @export
setCopyOnWrite <- function(x, value) {
    sharedObjectProperties(x, copyOnWrite = value)
}
#' @rdname sharedObjectProperties
#' @export
setSharedSubset <- function(x, value) {
    sharedObjectProperties(x, sharedSubset = value)
}
#' @rdname sharedObjectProperties
#' @export
setSharedCopy <- function(x, value) {
    sharedObjectProperties(x, sharedCopy = value)
}

context("cluster export")
sharedObjectPkgOptions(minLength = 1)
library(parallel)
cl = makeCluster(1)
clusterEvalQ(cl, {
    library(SharedObject)
})
n = 100
data = floor(abs(runif(n) * 2))
type = c(
    logical = as.logical,
    integer = as.integer,
    double = as.double,
    raw = as.raw
)
typeName = names(type)

for (i in seq_along(typeName)) {
    test_that(paste0("Testing basic subset for type ", typeName[i]), {
        curData = type[[i]](data)
        so = share(curData)
        expect_equal(curData, so)
        expect_equal(is.shared(so),TRUE)
        expect_equal(is.shared(so[1:2]), sharedObjectPkgOptions("sharedSubset"))
    })
    gc()
    test_that(paste0("Testing duplicate for type ", typeName[i]), {
        if (i <= 3) {
            curData = type[[i]](data)
            sv = share(curData, copyOnWrite  = FALSE)
            sv2 = sv
            sv2[1] = type[[i]](1 - sv[1])
            expect_equal(sv, sv2)

            sv = share(curData, copyOnWrite = TRUE)
            sv2 = sv
            sv2[1] = type[[i]](1 - sv2[1])
            expect_false(curData[1] == sv2[1])
        }else{
            expect_true(TRUE)
        }
    })
    gc()
    test_that(paste0("Testing cluster export for type ", typeName[i]), {
        curData = type[[i]](data)
        sv = share(curData)
        clusterExport(cl, "sv", envir = environment())
        res = clusterEvalQ(cl, {
            sv
        })
        expect_equal(curData, res[[1]])
        ## ownership
        res = clusterEvalQ(cl, {
            sharedObjectProperties(sv)$ownData
        })
        expect_equal(res[[1]], FALSE)
    })

    gc()
    test_that(paste0(
        "Testing copy-on-write for cluster export for type ",
        typeName[i]
    ),
    {
        if (i <= 3) {
            curData = type[[i]](data)
            sv = share(curData)
            setCopyOnWrite(sv, FALSE)
            clusterExport(cl, "sv", envir = environment())

            ## CopyOnWrite property
            res <- clusterEvalQ(cl, {
                getCopyOnWrite(sv)
            })
            expect_equal(res[[1]], FALSE)

            ## Test behavior
            res <- clusterEvalQ(cl, {
                sv[1] <- as(1 - sv[1], typeof(sv))
                sv
            })
            expect_equal(0 + sv[1], 1 - data[1])
        }else{
            expect_true(TRUE)
        }
    })
    gc()
}
stopCluster(cl)
gc()

context("package options")
sharedObjectPkgOptions(minLength = 1)

test_that("Shared Object Global options", {
    old <- sharedObjectPkgOptions()
    sharedObjectPkgOptions(
        copyOnWrite = FALSE,
        sharedSubset = FALSE,
        sharedCopy = FALSE
    )

    x = share(runif(10))
    # sharedObjectProperties(x)
    expect_equal(getCopyOnWrite(x), FALSE)
    expect_equal(getSharedCopy(x), FALSE)
    expect_equal(getSharedSubset(x), FALSE)

    setCopyOnWrite(x, TRUE)
    setSharedCopy(x, FALSE)
    setSharedSubset(x, TRUE)
    expect_equal(getCopyOnWrite(x), TRUE)
    expect_equal(getSharedCopy(x), FALSE)
    expect_equal(getSharedSubset(x), TRUE)

    do.call(sharedObjectPkgOptions,old)
})
gc()

test_that("Option: mustWork", {
    old <- sharedObjectPkgOptions()
    ## Use the default setting, expect error
    sharedObjectPkgOptions(mustWork = TRUE)
    data <- list(a= 1:10,b=as.symbol("a"))
    expect_error(so <- share(data))
    expect_error(so <- tryShare(data), NA)


    ## Temporary overwrite the setting, expect no error
    expect_error(so <- share(data, mustWork = FALSE), NA)
    expect_error(so <- tryShare(data), NA)

    ## Overwrite global setting, expect no error
    sharedObjectPkgOptions(mustWork = FALSE)
    expect_error(so <- share(data), NA)
    expect_error(so <- tryShare(data), NA)

    ## Temporary overwrite the setting, expect error
    expect_error(so <- share(data, mustWork = TRUE))
    expect_error(so <- tryShare(data), NA)

    do.call(sharedObjectPkgOptions,old)
})

context("Subset shared vectors")

test_that("normalized and out-of-range indices match base R", {
    value <- as.integer(101:110)
    indices <- list(
        negative_and_zero = -3:0,
        upper_out_of_range = 5:(length(value) + 1L),
        nonconsecutive_duplicates = c(8L, 2L, 8L, 1L),
        zeros_mixed_positive = c(0L, 5L, 0L, 2L),
        missing_and_out_of_range = c(NA_integer_, 1L, 11L)
    )

    for (sharedSubset in c(FALSE, TRUE)) {
        shared <- share(value, sharedSubset = sharedSubset)
        for (index in indices) {
            result <- shared[index]
            expect_identical(result, value[index])
            expect_identical(is.shared(result), sharedSubset)
        }
    }

    gc()
})

---
title: "新手指引"
author: 
- name: Jiefei Wang
  affiliation: Roswell Park Comprehensive Cancer Center, Buffalo, NY
date: "`r Sys.Date()`"
output:
    BiocStyle::html_document:
        toc: true
        toc_float: true
vignette: >
  %\VignetteIndexEntry{quickStartChinese}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  package: SharedObject
---

```{r setup, include = FALSE}
# knitr::knit("vignettes/quick_start_guide.Rmd", output = "README.md")
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library("SharedObject")
SharedObject:::setVerbose(FALSE)
```
# 介绍
在R的多进程并行运算中，如果每个进程需要读取同一个数据，常见的做法就是将数据读入每个进程内，然后再进行运算。虽然这种方法简单直接，但是对于只读数据来说，这是一种极大的内存浪费。举个例子，在一个常见的4核8线程的计算机上面，如果进行8个进程的并行运算，需要读取的数据为1GB， 那么总共就需要8GB的内存用以加载数据。在生物学领域，这种问题会更加严重，像高通量测序动辄几个GB的大小，如果希望将数据全部读入内存，那么需要的配置将会非常昂贵。

`SharedObject` 是一个用来解决并行运算内存问题的package， 它可以将一个R对象的数据存放在共享内存中，使得多个进程可以读取同一个数据，因此只要数据可以放入内存，无论使用多少R进程运算都不会增加内存负担。这样可以极大减少并行运算中内存的瓶颈。

# 基础用法
## 通过现有的对象创建共享对象
如果希望以现有的R对象为模版创建共享对象，你只需要调用`share`函数，并且将R对象作为参数传入即可。在下面的例子中，我们将创建一个3*3的矩阵`A1`，然后调用`share`函数创建一个共享对象`A2`
```{r}
## Create data
A1 <- matrix(1:9, 3, 3)
## Create a shared object
A2 <- share(A1)
```
对于R使用者来说，`A1`和`A2`是完全一样的。所有可以用于`A1`的代码都可以无缝衔接到`A2`上面，我们可以检查`A1`和`A2`的数据类型
```{r}
## Check the data
A1
A2

## Check if they are identical
identical(A1, A2)
```
用户可以将`A2`当作一个普通的矩阵来使用。如果需要区分共享对象的话，可以使用`is.shared`函数
```{r}
## Check if an object is shared
is.shared(A1)
is.shared(A2)
```
我们知道R里面有许多并行运算的package，例如`parallel`和`BiocParallel`。你可以使用任何package来传输共享对象`A2`，在下面的例子中我们使用最基础的`parallel` package来传输数据。
```{r}
library(parallel)
## Create a cluster with only 1 worker
cl <- makeCluster(1)
clusterExport(cl, "A2")
## Check if the object is still a shared object
clusterEvalQ(cl, SharedObject::is.shared(A2))
stopCluster(cl)
```
当你传输一个共享对象的时候，实际上只有共享内存的编号，还有一些R对象的信息被传输过去了。我们可以通过`serialize`函数来验证这一点
```{r}
## make a larger vector
x1 <- rep(0, 10000)
x2 <- share(x1)

## This is the actual data that will
## be sent to the other R workers
data1 <-serialize(x1, NULL)
data2 <-serialize(x2, NULL)

## Check the size of the data
length(data1)
length(data2)
```
通过查看被传输的数据，我们可以看到`x2`是明显小于`x1`的。当其他R进程接受到数据后，他们并不会为`x2`的数据分配内存，而是通过共享内存的编号来直接读取`x2`数据。因此，内存使用量会明显减少。

## 创建空的共享对象
和`vector`函数相似，你也可以直接创建一个空的共享对象
```{r}
SharedObject(mode = "integer", length = 6)
```
在创建共享对象过程中，你可以将对象的attributes直接给出
```{r}
SharedObject(mode = "integer", length = 6, attrib = list(dim = c(2L, 3L)))
```
如果需要了解更多细节，请参考`?SharedObject`

## 共享对象的属性
共享对象的内部结构里面有许多属性，你可以直接通过`sharedObjectProperties`来查看它们
```{r}
## get a summary report
sharedObjectProperties(A2)
```

`dataId`是共享内存的编号, `length`是共享对象的长度， `totalSize`是共享对象的大小, `dataType`是共享对象的数据类型, `ownData`决定了是否在当前进程内共享对象不需要使用的时候回收共享内存. `copyOnWrite`，`sharedSubset`和`sharedCopy` 决定了共享对象数据写入，取子集，和复制时候的行为. 我们将会在`package默认设置`和`进阶教程`里面详细讨论这三个参数.

需要注意的是，大部分共享对象的属性是不可变更的， 只有 `copyOnWrite`，`sharedSubset`和`sharedCopy` 是可变的. 你可以通过`getCopyOnWrite`，`getSharedSubset`和`getSharedCopy` 去得到一个共享对象的属性，也可以通过`setCopyOnWrite`，`setSharedSubset`和`setSharedCopy`去设置他们
```{r}
## get the individual properties
getCopyOnWrite(A2)
getSharedSubset(A2)
getSharedCopy(A2)

## set the individual properties
setCopyOnWrite(A2, FALSE)
setSharedSubset(A2, TRUE)
setSharedCopy(A2, TRUE)

## Check if the change has been made
getCopyOnWrite(A2)
getSharedSubset(A2)
getSharedCopy(A2)
```

# 支持的数据类型和结构
对于基础R类型来说，`SharedObject`支持`raw`，`logical`，`integer`，`numeric`，`complex`和`character`. 需要注意的是，共享字符串向量并不一定能够保证减少内存使用，因为字符串在R中有自己的缓存池，所以在传输字符向量串的时候我们仍然需要传输单个字符串，因此共享字符串向量只有在字符串重复次数比较多的时候会比较节约内存。因为字符串向量的特殊性，你也不能把字符串向量里面的字符串更改为一个从来没有在字符串向量里面出现过的字符串。

对于容器类型，`SharedObject`支持`list`，`pairlist`和`environment`。共享容器类型数据只会将容器内部的元素共享，容器本身并不会被共享，因此，如果你尝试向共享容器里面添加或删除元素，其他R进程是无法观测到你的修改的。因为`data.frame`本质上是一个`list`，因此它也符合上述规则。

对于S3和S4类型来说，通常你可以直接共享S3/S4对象的数据。如果你希望共享的S3/S4对象非常特殊，例如它需要读取磁盘数据，`share`函数本身是一个S4的generic， 你可以通过重载函数来定义你自己的共享方法。

如果一个对象的数据结构并不支持被共享，`share`函数将会直接返回原本的对象。这只会在很特殊情况发生，因为`SharedObject`包支持大部分数据类型。如果你希望在无法共享的情况下返回一个异常，你可以在使用`share`时传入参数`mustWork = TRUE`。
```{r}
## the element `A` is sharable and `B` is not
x <- list(A = 1:3, B = as.symbol("x"))

## No error will be given, 
## but the element `B` is not shared
shared_x <- share(x)

## Use the `mustWork` argument
## An error will be given for the non-sharable object `B`
tryCatch({
  shared_x <- share(x, mustWork = TRUE)
},
error=function(msg)message(msg$message)
)
```
就像我们之前看到的一样，你可以使用`is.shared`去查看一个对象是否是共享对象。在默认的情况下，`is.shared`只会返回一个逻辑值，告诉你这个对象本身是否被共享了，或者它含至少一个共享对象。你可以通过传入`depth`参数来看到具体细节
```{r}
## A single logical is returned
is.shared(shared_x)
## Check each element in x
is.shared(shared_x, depth = 1)
```

# Package默认设置
package默认设置控制着默认情况下的共享对象的属性，你可以通过`sharedObjectPkgOptions`来查看它们
```{r}
sharedObjectPkgOptions()
```
就像我们之前讨论的一样，`mustWork = FALSE`意味着在默认情况下，当`share`函数遇到个不可共享的对象，它不会抛出任何异常而是直接返回对象本身。`sharedSubset` 决定了当你对一个共享对象取子集的时候，得到的子集是否是一个共享对象. `minLength`是共享对象最小的长度，当一个对象的长度小于最小长度的时候，它将不会被共享。

我们会在进阶章节里面讨论 `copyOnWrite` 和 `sharedCopy`，不过对于大部分用户来说，你并不需要关心它们。package的参数可以通过`sharedObjectPkgOptions`来更改
```{r}
## change the default setting
sharedObjectPkgOptions(mustWork = TRUE)

## Check if the change is made
sharedObjectPkgOptions("mustWork")

## Restore the default
sharedObjectPkgOptions(mustWork = FALSE)
```
需要注意的是，`share`函数的参数有着比package参数更高的优先级，因此你可以通过向`share`函数添加参数的方法来临时改变默认设置。例如，你可以通过`share(x, mustWork = TRUE)`来忽略package的默认`mustWork`设置。

# 进阶教程
## 写时拷贝
由于所有的R进程都会访问同一个共享内存的数据，如果在一个进程中更改了共享内存的数据，其他进程的数据也会受到影响。为了防止这种情况的发生，当一个进程试图修改数据内容的时候，共享对象将会被复制。举例来说
```{r}
x1 <- share(1:4)
x2 <- x1

## x2 becames a regular R object after the change
is.shared(x2)
x2[1] <- 10L
is.shared(x2)

## x1 is not changed
x1
x2
```
当我们尝试修改`x2`的时候，R首先会复制`x2`的数据，然后再修改它的值。因此，虽然`x1`和`x2`是同一个共享对象，对于`x2`的修改并不会影响`x1`的值。这个默认的行为可以通过`copyOnWrite`来进行更改
```{r}
x1 <- share(1:4, copyOnWrite = FALSE)
x2 <- x1

## x2 will not be duplicated when a change is made
is.shared(x2)
x2[1] <- 0L
is.shared(x2)

## x1 has been changed
x1
x2
```
当我们手动把`copyOnWrite`关闭的时候，修改`x2`会导致`x1`也被修改了。这个参数可以用于并行运算时写回数据，你可以提前分配好一个空的共享对象，关闭它的`copyOnWrite`，然后将它传给所有相关进程。当进程计算出结果后，直接将数据写回到共享对象中，这样子就不需要通过传统的数据传输方式将结果传回给主进程了。不过，需要注意的是，当我们关闭`copyOnWrite`的时候，你对共享对象的操作也可能导致意外的结果。举例来说
```{r}
x <- share(1:4, copyOnWrite = FALSE)
x
-x
x
```
仅仅是对于负数的调用，就会导致`x`的值被更改。因此，用户需要小心使用这个功能。写时拷贝可以通过`share`函数的`copyOnWrite`参数来设置，也可以通过`setCopyOnwrite`函数随时打开或者关闭

```{r}
## Create x1 with copy-on-write off
x1 <- share(1:4, copyOnWrite = FALSE)
x2 <- x1
## change the value of x2
x2[1] <- 0L
## Both x1 and x2 are affected
x1
x2

## Enable copy-on-write
## x2 is now independent with x1
setCopyOnWrite(x2, TRUE)
x2[2] <- 0L
## only x2 is affected
x1
x2
```


### 警告
如果你在尝试修改共享对象的时候，将一个高精度的值赋给一个低精度的共享对象上，R会自动进行数据类型转换，将低精度的共享对象变为一个高精度的对象，因此，你实际上修改的将是高精度的普通对象而不是共享对象，即便你将写时拷贝关闭掉，你对它的修改也不会被其他R进程所共享。所以，当你尝试修改一个共享对象时，你需要特别小心共享对象所使用的数据类型。

## 共享拷贝
`sharedCopy` 参数决定了一个共享对象的拷贝是否仍然是一个共享对象。举例来说
```{r}
x1 <- share(1:4)
x2 <- x1
## x2 is not shared after the duplication
is.shared(x2)
x2[1] <- 0L
is.shared(x2)


x1 <- share(1:4, sharedCopy = TRUE)
x2 <- x1
## x2 is still shared(but different from x1) 
## after the duplication
is.shared(x2)
x2[1] <- 0L
is.shared(x2)
```
由于性能上的考虑，默认的设置为`sharedCopy=FALSE`，不过你可以随时通过`setSharedCopy`来更改一个共享对象的设置。需要注意的是，`sharedCopy`只能够在`copyOnWrite = TRUE`的时候生效。

## 列出共享内存编号
你可以通过`listSharedObjects`函数来列出所有的共享对象使用的共享内存编号
```{r}
listSharedObjects()
```
对用户来说，这个函数并不会被经常使用，不过如果你遇到了共享内存泄漏的问题，你可以通过`freeSharedMemory(ID)`手动释放共享内存。

# 基于`SharedObject`开发package
我们提供了三个级别的函数库来帮助开发者开发新的package。

## 用户API
开发新package最简单的方法是通过重载`share`函数来支持更加多的数据类型。我们推荐基于已有的`share`功能来开发更加丰富的功能，这样`SharedObject`将会帮你管理所有的共享内存，你不需要手动管理内存的生命周期。

## R的共享内存管理API
如果你需要手动管理共享内存，你可以通过`SharedObject`中提供的`allocateSharedMemory`，`mapSharedMemory`，`unmapSharedMemory`，`freeSharedMemory`，`hasSharedMemory`和`getSharedMemorySize`来进行内存的申请和释放. 需要注意的事，如果你手动申请了一个共享内存，在你使用后你需要手动释放它，否则将会导致内存泄漏。

## C++的共享内存管理API
如果你需要使用C++开发package，你可能需要使用C++函数去管理共享内存。`SharedObject`中所有的功能你都可以通过package里面的C++函数来做到。下面是关于如何链接和使用`SharedObject`中C++ API的教程。

### 第一步
为了使用C++ API，你需要将`SharedObject`添加进DESCRIPTION文件中的LinkingTo条目里面
```
LinkingTo: SharedObject
```
### 第二步
在你的C++文件里，引用`SharedObject`的头文件`#include "SharedObject/sharedMemory.h"`。

### 第三步
为了编译和链接你的package， 你需要在src目录下添加个Makevars文件
```
SHARED_OBJECT_LIBS = $(shell echo 'SharedObject:::pkgconfig("PKG_LIBS")'|\
"${R_HOME}/bin/R" --vanilla --slave)
SHARED_OBJECT_CPPFLAGS = $(shell echo 'SharedObject:::pkgconfig("PKG_CPPFLAGS")'|\
"${R_HOME}/bin/R" --vanilla --slave)

PKG_LIBS := $(PKG_LIBS) $(SHARED_OBJECT_LIBS)
PKG_CPPFLAGS := $(PKG_CPPFLAGS) $(SHARED_OBJECT_CPPFLAGS)
```
需要注意的是`$(shell ...)`是个GNU make语法，因此你也需要把GNU make添加进DESCRIPTION文件中SystemRequirements条目
```
SystemRequirements: GNU make
```

你可以在`SharedObject`的头文件中找到关于它C++ API的使用说明。

# Session Information
```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SharedObject.R
\name{SharedObject}
\alias{SharedObject}
\title{Create an empty shared object}
\usage{
SharedObject(
  mode = c("raw", "logical", "integer", "numeric", "complex"),
  length,
  attrib = list(),
  ...
)
}
\arguments{
\item{mode}{the type of the shared object}

\item{length}{the length of the shared object}

\item{attrib}{the attributes of the shared object}

\item{...}{Parameters that is used to create the shared object, please
refer to \code{?share} for details.}
}
\value{
An R vector
}
\description{
Create an empty shared object with a specific length and attributes.
}
\examples{
## Create an empty shared vector
x1 <- SharedObject(mode = "numeric", length = 10)
x1
## Create an empty shared matrix
x2 <- SharedObject(mode = "numeric", length = 6,
                 attrib = list(dim = c(2L,3L)))
x2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allGenerics.R, R/share.R
\name{share}
\alias{share}
\alias{share,vector-method}
\alias{share,matrix-method}
\alias{share,data.frame-method}
\alias{share,list-method}
\alias{share,ANY-method}
\title{Create a shared object}
\usage{
share(x, ...)

\S4method{share}{ANY}(
  x,
  ...,
  copyOnWrite,
  sharedSubset,
  sharedCopy,
  sharedAttributes,
  mustWork,
  minLength
)
}
\arguments{
\item{x}{An R object that will be shared, see details.}

\item{...}{For generalization purpose.}

\item{copyOnWrite, sharedSubset, sharedCopy}{The parameters
controlling the behavior of a shared object, see details.}

\item{sharedAttributes}{Whether to share the attributes of the object \code{x}
(default \code{TRUE}). Note that attribute \code{class} and \code{names} will never be shared.}

\item{mustWork}{Whether to throw an error if \code{x} is not sharable
(e.g. x is a function).
This parameter has no effect on the object's attributes and S4 object.}

\item{minLength}{The minimum length of a shared object(default \code{3}).
If \code{length(x)} is smaller than the minimum length, it would not be shared.
This parameter can be used to reduce the memory fragmentation.}
}
\value{
A shared object
}
\description{
This function will create a shared object for the object \code{x}.
The behavior of the shared object is exactly the same as \code{x},
but the data of the shared object is allocated in the shared
memory space. Therefore, a shared object can be easily exported to the other
R workers without duplicating the data, which can reduce the memory consumption
and the overhead of data transmission.
}
\details{
The function returns a shared object corresponding to the argument \code{x} if it
is sharable. There should be no different between \code{x} and the return value except
that the latter one is shared. The attributes of \code{x} will also be shared if possible.

\strong{Supported types}

For the basic R type, the function supports \code{raw}, \code{logical},\code{integer},
\code{double}, \code{complex}. \code{character} can be shared, but sharing a \code{character}
is beneficial only when there are a lot repetitions in the
elements of the vector. Due to the complicated structure of the character
vector, you are not allowed to set the value of a shared
character vector to a value which haven't presented in the vector before.
It is recommended to treat a shared character vector as read-only.

For the container, the function supports \code{list}, \code{pairlist}
and \code{environment}. Note that sharing a container is equivalent
to share all elements in the container, the container itself
will not be shared.

The function \code{share} is an S4 generic. The default share method works for
most S3/S4 objects. Therefore, there is no need to define a S4 share method
for each S3/S4 class unless the S3/S4 class has a special implementation
(e.g. on-disk data).
The default method will share all slots the object contains and the object itself
if possible. No error will be given if any of these objects are not
sharable and they will be kept unchanged.

\strong{Behavior control}

The behavior of a shared object can be controlled through three parameters:
\code{copyOnWrite}, \code{sharedSubset} and \code{sharedCopy}.

\code{copyOnWrite} determines Whether a shared object needs to be duplicated when the
data of the shared object is changed. The default value is \code{TRUE},
but can be altered by passing \code{copyOnWrite = FALSE} to the function.
This parameter can be used to let workers directly write the result back to
a shared object.

Please note that the no-copy-on-write feature is not fully supported by R. When
\code{copyOnWrite} is \code{FALSE}, a shared object might not behaves as one expects.
Please refer to the example code to see the exceptions.

\code{sharedSubset} determines whether the subset of a shared object is still a shared object.
The default value is \code{FALSE}, and can be changed by passing \code{sharedSubset = TRUE}
to the function

At the time of writing, The shared subset feature will
cause an unnecessary memory duplication in R studio. Therefore, for
the performance consideration, it is recommended to keep the feature
off in R studio.

\code{sharedCopy} determines whether the object is still a shared object after the
duplication. Note that it must be used with \code{copyOnWrite = TRUE}. Otherwise,
the shared object will never be duplicated. The default value is \code{FALSE}.
}
\examples{
## For vector
x <- runif(10)
so <- share(x)
x
so

## For matrix
x <- matrix(runif(10), 2, 5)
so <- share(x)
x
so

## For data frame
x <- as.data.frame(matrix(runif(10), 2, 5))
so <- share(x)
x
so

## export the object
library(parallel)
cl <- makeCluster(1)
clusterExport(cl, "so")
## check the exported object in the other process
clusterEvalQ(cl, so)

## close the connection
stopCluster(cl)

## Copy on write
x <- runif(10)
so1 <- share(x, copyOnWrite = TRUE)
so2 <- so1
so2[1] <- 10
## so1 is unchanged since copy-on-write feature is on.
so1
so2

## No copy on write
so1 <- share(x, copyOnWrite = FALSE)
so2 <- so1
so2[1] <- 10
#so1 is changed
so1
so2

## Flaw of no-copy-on-write
## The following code changes the value of so1,
## highly unexpected! Please use with caution!
-so1
so1
## The reason is that the minus function tries to
## duplicate so1 object, but the duplication function
## will return so1 itself, so the values in so1 get changed.

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allGenerics.R
\name{unshare}
\alias{unshare}
\alias{unshare,ANY-method}
\alias{unshare,vector-method}
\alias{unshare,list-method}
\title{Unshare a shared object}
\usage{
unshare(x)
}
\arguments{
\item{x}{a shared object, or an object that contains a shared object.}
}
\value{
An unshared object
}
\description{
Unshare a shared object. There will be no effect if the
object is not shared.
}
\examples{
x1 <- share(1:10)
x2 <- unshare(x1)
is.shared(x1)
is.shared(x2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/developerAPIS.R
\name{getLastIndex}
\alias{getLastIndex}
\alias{allocateSharedMemory}
\alias{mapSharedMemory}
\alias{unmapSharedMemory}
\alias{freeSharedMemory}
\alias{hasSharedMemory}
\alias{getSharedMemorySize}
\alias{initialSharedObjectPackageData}
\alias{releaseSharedObjectPackageData}
\title{Functions to manipulate shared memory}
\usage{
getLastIndex()

allocateSharedMemory(size, name = "")

mapSharedMemory(id)

unmapSharedMemory(id)

freeSharedMemory(id)

hasSharedMemory(id)

getSharedMemorySize(id)

initialSharedObjectPackageData()

releaseSharedObjectPackageData()
}
\arguments{
\item{size}{The size of the shared memory that you want to allocate}

\item{name, id}{The name of the shared memory}
}
\value{
\code{getLastIndex}: An interger ID served as a hint of the last created shared memory ID.

\code{allocateSharedMemory}: character ID(s) that can be used to find the shared memory

\code{mapSharedMemory}: External pointer(s) to the shared memory

\code{unmapSharedMemory}: No return value

\code{freeSharedMemory}: No return value

\code{hasSharedMemory}: Logical value(s) indicating whether the shared memory exist

\code{getSharedMemorySize}: A numeric value
}
\description{
These functions are designed for package developers only, they can
allocate, open, close and destroy shared memory without touching
C++ code. Normal users should not use these functions unless
dealing with memory leaking
}
\details{
\strong{Quick explanation}

\code{getLastIndex}: the ID of the last created shared memory.

\code{allocateSharedMemory}: allocate a shared memory of a given size,
the memory ID is returned by the function

\code{mapSharedMemory}: map the shared memory to the current process memory space

\code{unmapSharedMemory}: unmap the shared memory(without destroying it)

\code{freeSharedMemory}: free the shared memory. This function will only unmap the
shared memory on Windows.

\code{hasSharedMemory}: whether the memory exist?

\code{getSharedMemorySize}: get the actual size of the shared memory, it may be larger than the
size that you required.

\strong{Details}

A complete lifecycle of a shared memory involves four steps:
allocating, mapping, unmapping and freeing the shared memory.

The shared memory can be created by \code{allocateSharedMemory}.
The function \code{allocateSharedMemory} will return the ID of the shared memory.
After creating the shared memory, it can be mapped to the current process by
\code{mapSharedMemory}. The return value is an external pointer to the shared memory.
Once the shared memory is no longer needed, it can be unmapped and destroyed by
\code{unmapSharedMemory} and \code{freeSharedMemory} respectively.
}
\examples{
size <- 10L
## unnamed shared memory
id <- allocateSharedMemory(size)
hasSharedMemory(id)
ptr <- mapSharedMemory(id)
ptr
getSharedMemorySize(id)
unmapSharedMemory(id)
freeSharedMemory(id)
hasSharedMemory(id)

## named shared memory
name <- "SharedObjectExample"
if(!hasSharedMemory(name)){
    allocateSharedMemory(size, name = name)
    hasSharedMemory(name)
    ptr <- mapSharedMemory(name)
    ptr
    getSharedMemorySize(name)
    unmapSharedMemory(name)
    freeSharedMemory(name)
    hasSharedMemory(name)
}
}
\seealso{
\code{\link{listSharedObjects}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allGenerics.R, R/sharedObjectProperty.R
\name{sharedObjectProperties}
\alias{sharedObjectProperties}
\alias{sharedObjectProperties,ANY-method}
\alias{sharedObjectProperties,list-method}
\alias{getCopyOnWrite}
\alias{getSharedSubset}
\alias{getSharedCopy}
\alias{setCopyOnWrite}
\alias{setSharedSubset}
\alias{setSharedCopy}
\title{Get/Set the properties of a shared object.}
\usage{
sharedObjectProperties(x, ..., literal = TRUE)

\S4method{sharedObjectProperties}{ANY}(x, ..., literal = TRUE)

\S4method{sharedObjectProperties}{list}(x, ..., literal = TRUE)

getCopyOnWrite(x)

getSharedSubset(x)

getSharedCopy(x)

setCopyOnWrite(x, value)

setSharedSubset(x, value)

setSharedCopy(x, value)
}
\arguments{
\item{x}{A shared object}

\item{...}{The name of the property(s), it can be either symbols or characters.
if the argument is missing, it means getting all properties. See examples.}

\item{literal}{Whether the parameters in \code{...} are always treated as characters.}

\item{value}{The value of the property}
}
\value{
get: The property(s) of a shared object

set: The old property(s)
}
\description{
Get/Set the properties of a shared object.
}
\details{
For numeric objects, the properties are \code{dataId}, \code{length},
\code{totalSize}, \code{dataType}, \code{ownData}, \code{copyOnWrite}, \code{sharedSubset},
\code{sharedCopy}.

For character objects, the properties are
\code{length}, \code{unitSize},\code{totalSize},\code{dataType},\code{uniqueChar},\code{copyOnWrite}.

Note that only \code{copyOnWrite}, \code{sharedSubset} and \code{sharedCopy} are mutable.
The other attributes are read-only.
}
\examples{
## For numeric objects
x1 <- share(1:10)

## Get attributes
sharedObjectProperties(x1)
sharedObjectProperties(x1, copyOnWrite)
sharedObjectProperties(x1, "copyOnWrite")
props <- "copyOnWrite"
sharedObjectProperties(x1, props, literal = FALSE)
getCopyOnWrite(x1)

## Set attributes
sharedObjectProperties(x1, copyOnWrite = FALSE)
setCopyOnWrite(x1, FALSE)

## For character objects
x2 <- share(letters)
sharedObjectProperties(x2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/developerAPIS.R
\name{listSharedObjects}
\alias{listSharedObjects}
\title{List all shared Objects}
\usage{
listSharedObjects(end = NULL, start = NULL)
}
\arguments{
\item{end}{the end value of the ID. The default is \code{NULL}. See details.}

\item{start}{the start value of the ID. The default is \code{NULL}. See details.}
}
\value{
A data.frame object with shared object id and size
}
\description{
List all shared Objects
}
\details{
The parameter \code{start} and \code{end} specify the range of the ID. If not specified, all
IDs will be listed.

On Ubuntu or some other linux systems, the shared objects
can be found in the folder \verb{/dev/shm}. The function can find all shared objects
if the folder exists.

On Windows, since there is no easy way to find all shared objects.
the function will guess the range of the shared object IDs and search
for all IDs within the range. Therefore, if there are too many shared
objects(over 4 billions) ,the object id can be out of the searching range
and the result may not be complete.
Furthermore, there will be no named shared memory in the returned list.

Note that the size in the return value is the true memory size
that is reserved for the shared object, so it might be larger
than the object size.
}
\examples{
x <- share(runif(10))
## Automatically determine the search range
listSharedObjects()

## specify the search range
listSharedObjects(start = 10, end = 20)

## Search from 0 to 20
listSharedObjects(20)
}
\seealso{
\code{\link{getLastIndex}}, \code{\link{allocateSharedMemory}},
\code{\link{mapSharedMemory}}, \code{\link{unmapSharedMemory}},
\code{\link{freeSharedMemory}}, \code{\link{hasSharedMemory}}, \code{\link{getSharedMemorySize}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is.altrep}
\alias{is.altrep}
\title{Whether an object is an ALTREP object}
\usage{
is.altrep(x)
}
\arguments{
\item{x}{an R object}
}
\value{
A logical value
}
\description{
Whether an object is an ALTREP object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allGenerics.R, R/is.shared.R
\name{is.shared}
\alias{is.shared}
\alias{is.shared,ANY-method}
\title{Test whether an object is shared}
\usage{
is.shared(x, ..., depth = 0, showAttributes = FALSE)

\S4method{is.shared}{ANY}(x, ..., depth = 0, showAttributes = FALSE)
}
\arguments{
\item{x}{An R object}

\item{...}{For generalization purpose only}

\item{depth}{Whether to recursively check the element of \code{x}. This parameter
only works for container objects(e.g. \code{list} and \code{environment}), see details.}

\item{showAttributes}{Whether to check the attributes of \code{x}, default \code{FALSE}.}
}
\value{
a single logical value or a list.
}
\description{
Test whether an object is shared
}
\details{
When \code{depth=0}, the \code{is.shared} function return a single logical value indicating
whether \code{x} is shared or contains any shared objects. When \code{depth>0} and \code{x} is a
container(e.g. \code{list}), the function will recursively check each element of \code{x} and
return a list with each elements corresponding to the elements in \code{x}.
The \code{depth} parameter determines the depth of the checking procedure.

if \code{showAttributes = TRUE}, the attributes of the object will also be checked. The
check result is returned as attributes of the return value by appending \code{Shared} to the
end of the original attribute names. Note that \code{showAttributes} has
no effect on an S4 object for the attributes of an S4 object are used to store the
slots and should not be treated as the attributes of an object.
}
\examples{
x1 <- share(1:10)
is.shared(x1)

x2 <- share(list(a=1:10, b = list(d = letters, e = runif(10))))
is.shared(x2, depth=0)
is.shared(x2, depth=0, showAttributes = TRUE)
is.shared(x2, depth=1)
is.shared(x2, depth=2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/packageOptions.R
\name{sharedObjectPkgOptions}
\alias{sharedObjectPkgOptions}
\title{Get or set the global options for the SharedObject package}
\usage{
sharedObjectPkgOptions(..., literal = TRUE)
}
\arguments{
\item{...}{The name of the option(s), it can be either symbols or characters.
if the argument is missing, it means getting all option. See examples.}

\item{literal}{Whether the parameters in \code{...} are always treated as characters.}
}
\value{
\code{set}: The old package options

\code{get}: A list of the package options or a single value
}
\description{
Get or set the global options for the SharedObject package
}
\examples{
## Get all options
sharedObjectPkgOptions()

## Get copyOnWrite only
sharedObjectPkgOptions(copyOnWrite)
sharedObjectPkgOptions("copyOnWrite")
opt <- "copyOnWrite"
sharedObjectPkgOptions(opt, literal = FALSE)


## Set options
sharedObjectPkgOptions(copyOnWrite = FALSE)
## Check if we have changed the option
sharedObjectPkgOptions(copyOnWrite)

## Restore the default
sharedObjectPkgOptions(copyOnWrite = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{pkgconfig}
\alias{pkgconfig}
\title{Find path of the shared memory header file}
\usage{
pkgconfig(x)
}
\arguments{
\item{x}{Character, "PKG_LIBS" or "PKG_CPPFLAGS"}
}
\value{
path to the header or compiler flags
}
\description{
This function will return the path of the shared memory header or
the flags that are used to compile the package
for the developers who want to use C++ level implementation of the
\code{SharedObject} package
}
\examples{
SharedObject:::pkgconfig("PKG_LIBS")
SharedObject:::pkgconfig("PKG_CPPFLAGS")
}
